//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#ifndef EmissionParameters_h
#define EmissionParameters_h 1

#include "G4Material.hh"
#include "G4TransportationManager.hh"
#include "G4Navigator.hh"
#include "G4PathFinder.hh"
#include "G4RandomDirection.hh"
#include "ParametersSetting.hh"

class ParametersSetting;

class EmissionParameters
{
  public:

 EmissionParameters();
 ~EmissionParameters();
 
 void ConstructSpectrum(G4int ElementInd,G4int ParticleInd);
 G4double CalculateOneEmission_GetEnergy();
 G4double GetPositionX(){return PosX;}
 G4double GetPositionY(){return PosY;}
 G4double GetPositionZ(){return PosZ;}	

 static std::vector<G4double> InfMatrixE_Det;
 static std::vector<G4double> Eemitted_Probe;
	
  private:

 G4bool BeginOfRun;
 G4int DetVox, DetDef, VoxForEmiss, IntVoxForEmiss, N_Voxels, Obj_N_Voxels, Element, Elt, part, SpectDim, navigatorID, ProbeWorldID;
 G4int N_Vox[3], Obj_nVox[3]; 
 G4double Cpt, Cpt_Det, Int,Energy, PosX, PosY, PosZ, MassEmetter;
 G4double D_vox[3], D_DimDet[3], offset[3];
 G4String SpectData,IntData;
 std::vector<G4double> energy, proba, X, Y, Z, C_Elt_Mass, C_Elt_Densi_Det;        
	
};
#endif
