//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#ifndef ParametersSetting_h
#define ParametersSetting_h 1

#include "globals.hh"
#include "G4SystemOfUnits.hh"
#include <limits> 
#include <stdio.h>
#include <dirent.h>
#include <fstream>
#include <vector>
#include <map>

class ParametersSetting
{
  public:
 ParametersSetting();
 ~ParametersSetting();
 
 void SetStaticParamaters();

 static G4String ParamName; // name of the pilote file
 static G4String ResultName; // name for the results files

 // simulation parameters
 static G4double NbPart; static G4double clock;
 static G4int RadNat; static G4double MoDir[3];
 static G4int RadEl; static G4double CutValue;
 static G4int DetectorDef; static G4int DetectorVox; static G4int MatOfInterest; static G4bool RadioAGrain;
 static G4int NbProbeSegment; static G4double ProbeDiameter;
 static G4double ProbeOffSetXY[2];
 static G4int NbCompo;
 static G4int NbMat;

 // new components definition
 static std::vector<G4String> CompName;
 static std::vector<G4double> CompDensity;
 static std::vector<G4int> Comp_NbAtom;
 static std::vector<G4String> Comp_ListAtom;
 static std::vector<G4int> Comp_ListAtNumber;

 // material definition
 static std::vector<G4String> MatName;
 static std::vector<G4double> MatDensity;
 static std::vector<G4double> MatWaterContent;
 static std::vector<G4int> Mat_NbComp;
 static std::vector<G4int> Mat_ListComponents;
 static std::vector<G4double> Mat_ListContents;

 // grain packings definition
 static std::vector<G4int> CompGrainId; static std::vector<G4double> Graindensity;
 static std::vector<G4double> Compacity; static std::vector<G4int> NbLineGranu;
 static std::map<G4int,std::vector<G4double> > Diameter; static std::map<G4int,std::vector<G4double> > VolFraction;
 
 // voxel
 static G4int NbVox_byAxis[3]; static G4double DimVox[3]; static G4double DimSample[3];
 static G4int NbVoxels;

 // main geometry mapping
 static std::vector<G4int> MaterialMap;
 static std::vector<G4double> RadContentMap;
 static std::vector<G4int> XposId;static std::vector<G4int> YposId;static std::vector<G4int> ZposId;

 // sub-voxelised detector parameters 
 static G4int Object_NbVoxel;
 static G4int Object_voxelNumberXYZ[3];
 static std::vector<G4double> Object_RadContentMap;                                                   
 static std::vector<G4int> Object_MaterialMap;
 static std::vector<G4int> Obj_XposId;static std::vector<G4int> Obj_YposId;static std::vector<G4int> Obj_ZposId;

 // grain packing box parameters
 static G4double DimDet[3];
 static G4double DetOffSet[3];

  private:      
	
};
#endif
