//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#ifndef RPOrbsParam_h
#define RPOrbsParam_h 1

#include "G4VPVParameterisation.hh"
#include "G4VPhysicalVolume.hh"
#include "G4LogicalVolume.hh"
#include "G4Box.hh"
#include "G4Material.hh"
#include "G4Orb.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "G4IntersectionSolid.hh"
#include "RandomPacking.hh"

class RandomPacking;
class ParametersSetting;

class RPOrbsParam : public G4VPVParameterisation
{
 public:

  RPOrbsParam(G4int MatId);
  virtual ~RPOrbsParam();
  virtual void ComputeTransformation(const G4int copyNo,G4VPhysicalVolume *physVol) const;// position, rotation
  virtual G4Material* ComputeMaterial(const G4int copyNo,G4VPhysicalVolume* currentVol,// material, vis attributes
                                        const G4VTouchable* parentTouch=0);
  virtual G4VSolid* ComputeSolid(const G4int copyNo,G4VPhysicalVolume *physVol); // shape

  RandomPacking RandPack;
  G4int GetGrainPieceNumber(){return NbPieceGrain;}
  G4double GetDiameterAndDetector(G4int copyNo){return fDiametre[copyNo];}
  
 private:

  G4int NbPieceGrain,Ix,Iy,Iz;
  G4double fracVol,X,Y,Z,Diam,diametre,pdiametre,Xi,Yi,Zi,FVol,VolMoyGrain;
  std::vector<G4double> fDiametre, x, y, z;
  std::map<G4double, G4VisAttributes*> GrainColor;
  std::map<G4double, G4VisAttributes*>::iterator it_color;
  std::map<int, G4VSolid*> fSolMap;
  std::map<G4int, G4VSolid*>::iterator it_grain;

  G4Material* MatGrain;
  G4VSolid* sol;
  G4Box* voxel;
    
};

#endif


