//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#ifndef RandomPacking_h
#define RandomPacking_h 1

#include "Randomize.hh"
#include "ParametersSetting.hh"

class ParametersSetting;

//const G4float fracVol = 35./100.; // fraction volumique de grain

class RandomPacking
{
	public:

 RandomPacking();
 ~RandomPacking();
 
 void CalcRandomPacking(G4int mat);

 G4int GetGrainNumber(){return NbGrain;}
 G4int GetGrainPiecesNumber(){return NbPieceGrain;}
 G4int GetWholeGrainNumber(){return NbWholeGrain;};

 G4double GetGrainDiameter(G4int i){return fDiametreFinal[i];}
 G4double GetGrainPosX(G4int i){return xFinal[i];}
 G4double GetGrainPosY(G4int i) {return yFinal[i];}
 G4double GetGrainPosZ(G4int i) {return zFinal[i];}

 static std::vector<G4bool> IsValidDetector;
 
	private:
	
 G4double VoxeldX,VoxeldY,VoxeldZ,EchX,EchY,EchZ,fracVol,D,Dworld,densi,diam,prop,VolRelatif;

 G4int NbVoxel,X,Y,Z,NbGrain,NbPieceGrain,NbWholeGrain;

 G4String method;
 
 std::vector<G4double> Diam;
 std::vector<G4double> Prop;
 std::vector<G4double> FractionVol;
    
 std::vector<G4double> fDiametre;
 std::vector<G4double> x;
 std::vector<G4double> y;
 std::vector<G4double> z;

 std::vector<G4double> fDiametreFinal;
 std::vector<G4double> xFinal;
 std::vector<G4double> yFinal;
 std::vector<G4double> zFinal; 
 
};
#endif
