//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#ifndef RunAction_h
#define RunAction_h 1

#include "G4UserRunAction.hh"
#include "G4PhysicalConstants.hh"
#include "DetectorParallelWorld.hh"
#include "ParametersSetting.hh"
#include "EmissionParameters.hh"
#include <sstream>

class ParametersSetting;
//class DetectorParallelWorld;
class EmissionParameters;
class G4Run;

class RunAction : public G4UserRunAction
{
 public:
 
  RunAction();
  ~RunAction();

  void BeginOfRunAction(const G4Run*);
  void EndOfRunAction(const G4Run*);

  void GetInfiniteMatrixEnergy(G4double Energy){InfMatE += Energy;}
  void EnergyLoss(G4double Energy){Eloss += Energy;NbPartkilled ++;}
  void AccumProbeEnergy(G4double E,G4int i){ProbeE[i]+=E;
//if(i==50)G4cout<<"run "<<E/keV<<" "<<ProbeE[i]/keV<<G4endl;
}
  void AccumDetectorEnergy(G4double E,G4int i){Det1E[i]+=E;}
  void AccumGrainEnergy(G4double E,G4int DetNo,G4int GrainNo){DetE_2_3[DetNo][GrainNo]+=E;}
  void InitAtBeginOfRun(G4bool beginFlag){begin = beginFlag;}
  void WriteAtEndOfRun(G4bool endFlag){end = endFlag;}
    
 private:
  
  EmissionParameters* EmParam;

  G4int NbSegProb,nBox, NbPartkilled;
  G4double InfMatE, Eloss;
  G4bool begin,end;

  std::vector<G4double> ProbeE;
  std::vector<G4double>Det1E;
  std::vector<std::map<G4int, G4double> > DetE_2_3;

};

#endif
