//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#ifndef VoxParam_Opt_h
#define VoxParam_Opt_h 1

#include "G4VPVParameterisation.hh"
#include "G4VNestedParameterisation.hh"
#include "G4VTouchable.hh"
#include "G4VPhysicalVolume.hh"
#include "G4LogicalVolume.hh"
#include "G4Box.hh"
#include "G4Material.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "ParametersSetting.hh"

class ParametersSetting;
class G4VTouchable;

class VoxParam_Opt : public G4VNestedParameterisation
{
 public:

  VoxParam_Opt(G4int world);
  virtual ~VoxParam_Opt();

  virtual void ComputeTransformation(const G4int copyNo,G4VPhysicalVolume *physVol) const;// position, rotation
  virtual G4Material* ComputeMaterial(G4VPhysicalVolume* currentVol,const G4int copyNo,// material, vis attributes
                                        const G4VTouchable* parentTouch=0);
  virtual G4VSolid* ComputeSolid(const G4int copyNo,G4VPhysicalVolume *physVol); // shape
  virtual G4int GetNumberOfMaterials() const;
  virtual G4Material* GetMaterial(G4int idx) const{return Mat[idx];} // material listing

 private:

  G4int fNVoxX,fNVoxY, fNVoxZ, fWorldFlag;

  G4Material* SiO2;
  G4Material* fMat;
  G4Box *fVoxel, *fVoxelDet;

  std::vector<G4Material*> Mat;	  
  std::vector<G4VisAttributes*> Color;

  using G4VNestedParameterisation::ComputeMaterial;  
    
};

#endif


