//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "DetectorParallelWorld.hh"

using namespace std;

map<G4int,map<G4int,G4double> > DetectorParallelWorld::DiamVoxMap;

DetectorParallelWorld::DetectorParallelWorld(G4String& parallelWorldName):G4VUserParallelWorld(parallelWorldName),constructed(false)
{ 
 ;
}

DetectorParallelWorld::~DetectorParallelWorld()
{;}

void DetectorParallelWorld::Construct()
{
 if(constructed) return;
 constructed = true;

 //
 // Get the World
 //
 G4VPhysicalVolume* World = GetWorld();
 G4LogicalVolume* worldLog = World->GetLogicalVolume();

 //
 // mother of parallel world volumes
 //
 G4double DetWx = 1.5*ParametersSetting::DimSample[0];
 G4double DetWy = 1.5*ParametersSetting::DimSample[1];
 G4double DetWz = 1.5*ParametersSetting::DimSample[2]; // world dimensions

 paraMomSol = new G4Box("paraMom",DetWx/2.,DetWy/2.,DetWz/2.);
 paraMomLog = new G4LogicalVolume(paraMomSol,0,"paraMom");
 paraMomLog->SetOptimisation(true);
 paraMomPhy = new G4PVPlacement(0,G4ThreeVector(0.,0.,0.),paraMomLog,"paraMom",
                    worldLog,false,0);

 G4VisAttributes* ParaWorldVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
 ParaWorldVis->SetForceSolid(false);
 ParaWorldVis->SetVisibility(false);
 paraMomLog->SetVisAttributes(ParaWorldVis);

 //*************************** detector definition ***************************//

 //
 // Detector 1: sub-voxelised voxel detector
 //
 if(ParametersSetting::DetectorDef==1)
 {
  G4int NumVox = ParametersSetting::DetectorVox;
  G4double PosX = ParametersSetting::DimVox[0]*(1.*ParametersSetting::XposId[NumVox]-ParametersSetting::NbVox_byAxis[0]/2.+0.5);
  G4double PosY = ParametersSetting::DimVox[1]*(1.*ParametersSetting::YposId[NumVox]-ParametersSetting::NbVox_byAxis[1]/2.+0.5);
  G4double PosZ = ParametersSetting::DimVox[2]*(1.*ParametersSetting::ZposId[NumVox]-ParametersSetting::NbVox_byAxis[2]/2.+0.5);

  G4Box* Det1_Sol = new G4Box("Detector",ParametersSetting::DimVox[0]/2.,
                                         ParametersSetting::DimVox[1]/2.,
                                         ParametersSetting::DimVox[2]/2.);
  G4LogicalVolume* Det_Log = new G4LogicalVolume(Det1_Sol,0,"Detector");
  new G4PVPlacement(0,G4ThreeVector(PosX,PosY,PosZ),Det_Log ,"Detector",paraMomLog ,false,0);

  // X axis division 
  G4Box* DetDivX_Sol = new G4Box("DetDivX", ParametersSetting::DimDet[0]/2.,
                                            ParametersSetting::DimVox[1]/2.,
                                            ParametersSetting::DimVox[2]/2.);
  G4LogicalVolume* DetDivX_Log = new G4LogicalVolume(DetDivX_Sol,0,"DetDivX");
  VoxParamXY* ParamX = new VoxParamXY(0,1);
  new G4PVParameterised("DetDivX", DetDivX_Log,Det_Log, kXAxis,ParametersSetting::Object_voxelNumberXYZ[0], ParamX ,0);

  // Y axis division 
  G4Box* DetDivY_Sol = new G4Box("DetDivY", ParametersSetting::DimDet[0]/2.,
                                            ParametersSetting::DimDet[1]/2.,
                                            ParametersSetting::DimVox[2]/2.);
  G4LogicalVolume* DetDivY_Log = new G4LogicalVolume(DetDivY_Sol,0,"DetDivY");
  VoxParamXY* ParamY = new VoxParamXY(1,1);
  new G4PVParameterised("DetDivY", DetDivY_Log,DetDivX_Log, kYAxis,ParametersSetting::Object_voxelNumberXYZ[1], ParamY ,0);

  G4VisAttributes* DetDivVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
  DetDivVis->SetForceSolid(false);
  DetDivVis->SetVisibility(false);
  DetDivX_Log->SetVisAttributes(DetDivVis);
  DetDivY_Log->SetVisAttributes(DetDivVis);

  // Z axis voxelisation
  G4Box* IntVox_Sol = new G4Box("Internal_Voxel", ParametersSetting::DimDet[0]/2.,
                                                  ParametersSetting::DimDet[1]/2.,
                                                  ParametersSetting::DimDet[2]/2.);
  G4LogicalVolume* IntVox_Log = new G4LogicalVolume(IntVox_Sol,0,"Internal_Voxel");
  VoxParam_Opt* ParamVox = new VoxParam_Opt(1);
  new G4PVParameterised("Internal_Voxel", IntVox_Log,DetDivY_Log, kZAxis,ParametersSetting::Object_voxelNumberXYZ[2], ParamVox ,0);
  
  G4cout<<"successful creation of the sub-voxelised voxel"<<G4endl;
 }

 //
 // Detector 2: single random packing of grains
 //
 if(ParametersSetting::DetectorDef==2)
 {
  if((ParametersSetting::DimDet[0]/2.+fabs(ParametersSetting::DetOffSet[0])>ParametersSetting::DimVox[0]/2.)
    ||(ParametersSetting::DimDet[1]/2.+fabs(ParametersSetting::DetOffSet[1])>ParametersSetting::DimVox[1]/2.)
    ||(ParametersSetting::DimDet[2]/2.+fabs(ParametersSetting::DetOffSet[2])>ParametersSetting::DimVox[2]/2.))
  {G4cout<<" !! ******************************** !!" <<G4endl<<
           " !! detector 2 box exceeds its voxel !!" <<G4endl<<
           " !! ******************************** !!" <<G4endl;
   while(1)
   {
    G4cout<< "exit simulation? y/n"<<G4endl;
    G4String exit;
    G4cin>>exit;
    if(exit=="y")abort();
    if(exit=="n")break;
   }
  }
 
  G4int NumVox = ParametersSetting::DetectorVox;

  G4double PosX = ParametersSetting::DimVox[0]
                  *(1.*ParametersSetting::XposId[NumVox]-ParametersSetting::NbVox_byAxis[0]/2.+0.5)
                  +ParametersSetting::DetOffSet[0];                                                  
  G4double PosY = ParametersSetting::DimVox[1]
                 *(1.*ParametersSetting::YposId[NumVox]-ParametersSetting::NbVox_byAxis[1]/2.+0.5)
                   +ParametersSetting::DetOffSet[1];                                                  
  G4double PosZ = ParametersSetting::DimVox[2]
                  *(1.*ParametersSetting::ZposId[NumVox]-ParametersSetting::NbVox_byAxis[2]/2.+0.5)
                  +ParametersSetting::DetOffSet[2];                                                 

  G4Box* Det1_Sol = new G4Box("Detector",ParametersSetting::DimDet[0]/2.,ParametersSetting::DimDet[1]/2.,ParametersSetting::DimDet[2]/2.);
  G4String DetMatName = "NULL";
  if(ParametersSetting::Compacity[ParametersSetting::MaterialMap[NumVox]]<=0.)
  {G4cout<<"The material "<<ParametersSetting::MatName[ParametersSetting::MaterialMap[NumVox]]<<" "<<ParametersSetting::Compacity[NumVox]<<" filling in the detector voxel "<<NumVox<<
   " Does not contain any grain data. Please chose another voxel for detector or another detector type."<<G4endl;
   abort();}
  DetMatName = ParametersSetting::MatName[ParametersSetting::MaterialMap[NumVox]]+"_matrix";
  G4LogicalVolume* Det_Log = new G4LogicalVolume(Det1_Sol,G4Material::GetMaterial(DetMatName),"Detector");
  new G4PVPlacement(0,G4ThreeVector(PosX,PosY,PosZ),Det_Log ,"Detector",paraMomLog ,false,0);

  G4Orb* grain_Sol = new G4Orb("Grain",0.01*mm);
  G4LogicalVolume* grain_Log = new G4LogicalVolume(grain_Sol,0,"Grain",0,0,0);
  RPOrbsParam* RPgrains = new RPOrbsParam(ParametersSetting::MaterialMap[NumVox]);
  G4int NbPieceGrain = RPgrains->GetGrainPieceNumber();
  new G4PVParameterised("Grain", grain_Log,Det_Log, kUndefined, NbPieceGrain, RPgrains ,0);
 
  for(G4int i=0;i<NbPieceGrain;i++)
  {
   if(RPgrains->GetDiameterAndDetector(i)>0)DiamVoxMap[ParametersSetting::DetectorVox][i]= RPgrains->GetDiameterAndDetector(i);        
  }

  G4VisAttributes* GrainVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
  GrainVis->SetForceSolid(true);
  GrainVis->SetVisibility(true);
  //grain_Log->SetVisAttributes(GrainVis);

  G4cout<<"successful creation of the grain packing"<<G4endl;
 }

 //
 // Detector 3: successive random packings of grains in a column
 //
 if(ParametersSetting::DetectorDef==3)
 {
  if((ParametersSetting::DimDet[0]/2.+fabs(ParametersSetting::DetOffSet[0])>ParametersSetting::DimVox[0]/2.)
    ||(ParametersSetting::DimDet[1]/2.+fabs(ParametersSetting::DetOffSet[1])>ParametersSetting::DimVox[1]/2.))
  {
   G4cout<<" !! **************************************** !!" <<G4endl<<
           " !! detector 3 boxes exceeds its voxel column!!" <<G4endl<<
           " !! **************************************** !!" <<G4endl;
   while(1)
   {
    G4cout<< "exit simulation? y/n"<<G4endl;
    G4String exit;
    G4cin>>exit;
    if(exit=="y")abort();
    if(exit=="n")break;
   }      
  }

  map<G4int,RPOrbsParam*> ColParam;
  G4Orb *grainSol = new G4Orb("Grain",0.01*mm);

  G4int nZ =0;
  for(G4int i=0;i<ParametersSetting::NbVoxels;i++)
  {
   if((ParametersSetting::XposId[i]==ParametersSetting::XposId[ParametersSetting::DetectorVox])
     &&(ParametersSetting::YposId[i]==ParametersSetting::YposId[ParametersSetting::DetectorVox]))
   {
    std::ostringstream oss ;
    oss << i;
    G4String BoxName = "Detector" + oss.str();
    G4Box* Det_Sol = new G4Box(BoxName,ParametersSetting::DimDet[0]/2.,ParametersSetting::DimDet[1]/2.,ParametersSetting::DimDet[2]/2.);
    G4String DetMatName = "NULL";
    if(ParametersSetting::Compacity[ParametersSetting::MaterialMap[i]]>0)
    DetMatName=ParametersSetting::MatName[ParametersSetting::MaterialMap[i]]+"_matrix";
    else DetMatName = ParametersSetting::MatName[ParametersSetting::MaterialMap[i]];
    G4LogicalVolume* Det_Log = new G4LogicalVolume(Det_Sol,G4Material::GetMaterial(DetMatName),BoxName);

    for(G4int j=0;j< int(ParametersSetting::DimVox[2]/ParametersSetting::DimDet[2]);j++)
    {
     G4double PosX = ParametersSetting::DimVox[0]
                     *(1.*ParametersSetting::XposId[i]-ParametersSetting::NbVox_byAxis[0]/2.+0.5)
                     +ParametersSetting::DetOffSet[0];                                                
     G4double PosY = ParametersSetting::DimVox[1]
                     *(1.*ParametersSetting::YposId[i]-ParametersSetting::NbVox_byAxis[1]/2.+0.5)
                     +ParametersSetting::DetOffSet[1];                                            
     G4double PosZ = ParametersSetting::DimVox[2]
                     *(1.*ParametersSetting::ZposId[i]-ParametersSetting::NbVox_byAxis[2]/2.)
                     +(ParametersSetting::DimDet[2]*(j+0.5));
     new G4PVPlacement(0,G4ThreeVector(PosX,PosY,PosZ),Det_Log ,"Detector",paraMomLog ,false,nZ);
     nZ++;
    }

    if(ParametersSetting::Compacity[ParametersSetting::MaterialMap[i]]>0)
    {
     std::map<int, RPOrbsParam*>::iterator it = ColParam.find(ParametersSetting::MaterialMap[i]);
     if(it == ColParam.end())ColParam[ParametersSetting::MaterialMap[i]]= new RPOrbsParam(ParametersSetting::MaterialMap[i]);
     G4String RPName = "Grain" + oss.str();
     G4LogicalVolume* grainLog = new G4LogicalVolume(grainSol,0 ,RPName);
     G4int NbPieceGrain = ColParam[ParametersSetting::MaterialMap[i]]->GetGrainPieceNumber();
     for(G4int k=0;k<NbPieceGrain;k++)
     {
      if(ColParam[ParametersSetting::MaterialMap[i]]->GetDiameterAndDetector(k)>0)
      {
       DiamVoxMap[i][k]=ColParam[ParametersSetting::MaterialMap[i]]->GetDiameterAndDetector(k);
      }
     }
     new G4PVParameterised(RPName, grainLog,Det_Log, kUndefined, NbPieceGrain, ColParam[ParametersSetting::MaterialMap[i]] ,0);

     G4VisAttributes* GrainVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
     GrainVis->SetForceSolid(true);
     GrainVis->SetVisibility(true);
    }
   } // if the voxel is in the detector column	
  } // loop on all the voxel numbers

 G4cout<<"successful creation of the grain packing sequence"<<G4endl;
 }

;
}

