//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "EmissionParameters.hh"

using namespace std;

vector<G4double> EmissionParameters::InfMatrixE_Det;
vector<G4double> EmissionParameters::Eemitted_Probe;

EmissionParameters::EmissionParameters()
{
 // variable initialization
 VoxForEmiss=0;
 IntVoxForEmiss=0;
 Cpt =0.;
 Cpt_Det=0.;
 BeginOfRun = true;
 Elt=0; part =0;
 MassEmetter =0;
 PosX = 0;PosY = 0;PosZ = 0;

 // getting the static parameter values for faster access
 DetVox = ParametersSetting::DetectorVox;
 DetDef = ParametersSetting::DetectorDef;
 N_Voxels = ParametersSetting::NbVoxels;
 Obj_N_Voxels = ParametersSetting::Object_NbVoxel;
 D_vox[0] = ParametersSetting::DimVox[0];    D_vox[1] = ParametersSetting::DimVox[1];    D_vox[2] = ParametersSetting::DimVox[2];
 D_DimDet[0] = ParametersSetting::DimDet[0];       D_DimDet[1] = ParametersSetting::DimDet[1];       D_DimDet[2] = ParametersSetting::DimDet[2];
 offset[0] = ParametersSetting::DetOffSet[0];   offset[1] = ParametersSetting::DetOffSet[1];   offset[2] = ParametersSetting::DetOffSet[2];
 N_Vox[0] = ParametersSetting::NbVox_byAxis[0];     N_Vox[1] = ParametersSetting::NbVox_byAxis[1];     N_Vox[2] = ParametersSetting::NbVox_byAxis[2];
 Obj_nVox[0] = ParametersSetting::Object_voxelNumberXYZ[0];
 Obj_nVox[1] = ParametersSetting::Object_voxelNumberXYZ[1];
 Obj_nVox[2] = ParametersSetting::Object_voxelNumberXYZ[2];
 Eemitted_Probe.reserve(ParametersSetting::NbProbeSegment);
 for(G4int i=0;i<ParametersSetting::NbProbeSegment;i++)Eemitted_Probe[i]=0.;
 if(ParametersSetting::DetectorDef==1)InfMatrixE_Det.reserve(ParametersSetting::Object_NbVoxel);
 if(ParametersSetting::DetectorDef==2)InfMatrixE_Det.reserve(1);
 if(ParametersSetting::DetectorDef==3)InfMatrixE_Det.reserve(ParametersSetting::NbVox_byAxis[2]);
 for(G4int i=0;i<int(InfMatrixE_Det.capacity());i++)InfMatrixE_Det[i]=0.;
}

EmissionParameters::~EmissionParameters()
{;}

void EmissionParameters::ConstructSpectrum(G4int element,G4int Particle)
{
 Elt=element;
 part = Particle;

 //
 // Selection of the spectrum file
 //
 if(element==0) // user defined spectrum
 {
  SpectData = "data/spectra/UserDef";
 }
 if(element==1) // uranium series
 {
  if(Particle == 1)SpectData = "data/spectra/Ualpha"; // alpha
  if(Particle == 2)SpectData = "data/spectra/Ubeta"; // beta
  if(Particle == 3)SpectData = "data/spectra/Ugamma"; // gamma
 }
 if(element==2) // thorium series
 {
  if(Particle == 1)SpectData ="data/spectra/Thalpha"; // alpha
  if(Particle == 2)SpectData ="data/spectra/Thbeta"; // beta
  if(Particle == 3)SpectData = "data/spectra/Thgamma"; // gamma
 }
 if(element==3) // potassium 40
 {
  if(Particle == 2)SpectData ="data/spectra/Kbeta"; // beta
  if(Particle == 3)SpectData = "data/spectra/Kgamma"; // gamma
 }

 //
 // data reading and construction of the energy emission spectrum
 //
 G4float E,P;
 G4String ColumnName;
 G4int Sdim =0;   
 ifstream spect;

 spect.open(SpectData,ios::in);
 if(!spect.is_open()){G4cout<<"impossible to open the emission spectrum"<<G4endl;abort();}
 Sdim = std::count(std::istreambuf_iterator<char>( spect ),std::istreambuf_iterator<char>(),'\n' );
 spect.close();
 
 energy.reserve(Sdim);energy[0]=0;
 proba.reserve(Sdim);proba[0]=0;

 spect.open(SpectData,ios::in);
 spect>>ColumnName>>ColumnName;spect.ignore(100,'\n');
 for(G4int i=1; i<Sdim;i++)
 {
  spect>>E>>P;spect.ignore(100,'\n');
  energy[i]=E;
  proba[i]=P;
 }
 spect.close();
 SpectDim = Sdim-1;

 //
 // construction of the activity map of the sub-voxelised detector
 //
 G4double fSubVoxDet_act = 0.;
 if(DetDef==1)
 {
  C_Elt_Densi_Det.reserve(Obj_N_Voxels+1);C_Elt_Densi_Det[0]=0.;
  for(G4int i=1; i<Obj_N_Voxels+1; i++)
  {

   G4double density = ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[i-1]]*g/cm3;	
   fSubVoxDet_act += ((ParametersSetting::Object_RadContentMap[i-1])* density); 
   C_Elt_Densi_Det[i]= fSubVoxDet_act;
  }
  fSubVoxDet_act *= ParametersSetting::DimDet[0]*ParametersSetting::DimDet[1]*ParametersSetting::DimDet[2];
  
 }

 //
 // construction of the activity map of the main voxel grid
 //
 C_Elt_Mass.reserve(N_Voxels+1); C_Elt_Mass[0]=0;
 G4double fVoxCumulAct =0.;
 for(G4int i=1; i<N_Voxels+1; i++)
 {
  if((DetDef==1)&&(DetVox+1==i))fVoxCumulAct += fSubVoxDet_act; 
  else fVoxCumulAct += ParametersSetting::RadContentMap[i-1]
                       *ParametersSetting::MatDensity[ParametersSetting::MaterialMap[i-1]]*g/cm3
                       *ParametersSetting::DimVox[0]*ParametersSetting::DimVox[1]*ParametersSetting::DimVox[2];
  C_Elt_Mass[i]= fVoxCumulAct;
 }

} // end of spectrum and activity map construction

G4double EmissionParameters::CalculateOneEmission_GetEnergy()
{
 //
 // definition of the energy for the particle emission
 //
 G4double j = G4UniformRand();
 G4int k_min = 0;
 G4int k_max = SpectDim;
 G4int k = 0;
 Energy=0;
 
 while(k_max-k_min>1)
 { 
  k = (int) ((k_min+k_max)/2.) ;
  if (proba[k] < j){k_min = k;}
  else {k_max = k;}
 }
 Energy=energy[k_max]*keV;

 //
 // definition of the position of the particle emission
 //

 	// definition of the voxel of emission

 PosX = 0;PosY = 0;PosZ = 0;
 while(C_Elt_Mass[VoxForEmiss]/C_Elt_Mass[N_Voxels]<= Cpt/ParametersSetting::NbPart) VoxForEmiss ++; // incrementing of the voxel of emission
 k_max = VoxForEmiss-1;

 G4ThreeVector position(0.,0.,0.);
 G4ThreeVector momDirection(0.,0.,0.);

 PosX = D_vox[0]*(1. * ParametersSetting::XposId[k_max]-N_Vox[0]/2.) + D_vox[0]*G4UniformRand();
 PosY = D_vox[1]*(1. * ParametersSetting::YposId[k_max]-N_Vox[1]/2.) + D_vox[1]*G4UniformRand();
 PosZ = D_vox[2]*(1. * ParametersSetting::ZposId[k_max]-N_Vox[2]/2.) + D_vox[2]*G4UniformRand(); // random position inside the voxel of emission

 if(DetDef>1)
 {
  static G4Navigator* pNavigator = 0;
  if(!pNavigator)
  {
   G4String paraWName = "DetectorWorld"; 
   G4VPhysicalVolume* pPW =G4TransportationManager::GetTransportationManager()->GetParallelWorld(paraWName);
   pNavigator =G4TransportationManager::GetTransportationManager()->GetNavigator(pPW);
  } // finding the parallel world of detectors
  navigatorID = G4TransportationManager::GetTransportationManager()->ActivateNavigator(pNavigator);
 }
 static G4Navigator* ProbeNavigator = 0;
 static G4PathFinder* pPathFinder = G4PathFinder::GetInstance();// initialisation of the geometry navigation functions

  // tirage d'une position dans le voxel

 if((DetDef==1)&&(DetVox==k_max)) // in the case of a sub-voxelised detector
 {
  G4int k_IntVox =0;
  G4double NbPart_DetVox = ParametersSetting::NbPart*(C_Elt_Mass[DetVox+1]-C_Elt_Mass[DetVox])/C_Elt_Mass[N_Voxels];

  while(C_Elt_Densi_Det[IntVoxForEmiss]/C_Elt_Densi_Det[Obj_N_Voxels]<= Cpt_Det/NbPart_DetVox) 
  IntVoxForEmiss ++; // incrementing of the sub-voxel of emission
  k_IntVox = IntVoxForEmiss-1;
  Cpt_Det +=1.; // incrementing of the number of particle emitted in the sub-voxelised detector

  InfMatrixE_Det[k_IntVox] += Energy;

  PosX = D_vox[0]*(1.*ParametersSetting::XposId[k_max]-N_Vox[0]/2.+0.5)
         +D_DimDet[0]*(1.*ParametersSetting::Obj_XposId[k_IntVox]-Obj_nVox[0]/2.)
         +D_DimDet[0]*G4UniformRand();
  PosY = D_vox[1]*(1.*ParametersSetting::YposId[k_max]-N_Vox[1]/2.+0.5)
         +D_DimDet[1]*(1.*ParametersSetting::Obj_YposId[k_IntVox]-Obj_nVox[1]/2.)
         +D_DimDet[1]*G4UniformRand();
  PosZ = D_vox[2]*(1.*ParametersSetting::ZposId[k_max]-N_Vox[2]/2.+0.5)
         +D_DimDet[2] *(1.*ParametersSetting::Obj_ZposId[k_IntVox]-Obj_nVox[2]/2.)
         +D_DimDet[2] *G4UniformRand(); 
 }

 if((DetDef==2)&&(DetVox==k_max)) // in the case of a single grain packing detector
 { 
  InfMatrixE_Det[0] += Energy; // accumulate the energy in the detector map of energy deposited
  position = G4ThreeVector(PosX,PosY,PosZ);
  pPathFinder->PrepareNewTrack(position,momDirection);
  G4TouchableHandle touchableParaW = pPathFinder-> CreateTouchableHandle(navigatorID);

  while(!touchableParaW->GetVolume())
  { position = G4ThreeVector(PosX + (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.,
                             PosY + (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.,
		             PosZ+ (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.);
  } // avoiding the sample edge

  if(!ParametersSetting::RadioAGrain) // non-radioactive grains, radioactive matrix
  {
   while(touchableParaW->GetVolume()->GetName()=="Grain") // random position selection until a point in the matrix is find
   {
    PosX = D_vox[0] *(1.*ParametersSetting::XposId[k_max]-N_Vox[0]/2.+0.5)+ offset[0] + D_DimDet[0]*(G4UniformRand()-0.5);
    PosY = D_vox[1] *(1.*ParametersSetting::YposId[k_max]-N_Vox[1]/2.+0.5)+ offset[1] + D_DimDet[1]*(G4UniformRand()-0.5);
    PosZ = D_vox[2] *(1.*ParametersSetting::ZposId[k_max]-N_Vox[2]/2.+0.5)+ offset[2] + D_DimDet[2]*(G4UniformRand()-0.5);
    position = G4ThreeVector(PosX,PosY,PosZ);
    pPathFinder->PrepareNewTrack(position,momDirection);
    touchableParaW = pPathFinder-> CreateTouchableHandle(navigatorID);
   } 
  }
  else // radioactive grains, non-radioactive matrix
  {
   while(touchableParaW->GetVolume()->GetName()=="Detector") // random position selection until a point in the grains is find
   {
    PosX = D_vox[0] *(1.*ParametersSetting::XposId[k_max]-N_Vox[0]/2.+0.5)+ offset[0] + D_DimDet[0]*(G4UniformRand()-0.5);
    PosY = D_vox[1] *(1.*ParametersSetting::YposId[k_max]-N_Vox[1]/2.+0.5)+ offset[1] + D_DimDet[1]*(G4UniformRand()-0.5);
    PosZ = D_vox[2] *(1.*ParametersSetting::ZposId[k_max]-N_Vox[2]/2.+0.5)+ offset[2] + D_DimDet[2]*(G4UniformRand()-0.5);
    position = G4ThreeVector(PosX,PosY,PosZ);
    pPathFinder->PrepareNewTrack(position,momDirection);
    touchableParaW = pPathFinder-> CreateTouchableHandle(navigatorID);
   } 
  }
 }

 if((DetDef==3)
   &&(ParametersSetting::XposId[k_max]==ParametersSetting::XposId[DetVox])
   &&(ParametersSetting::YposId[k_max]==ParametersSetting::YposId[DetVox])) // in the case of a successive grain packings detector
 {
  position = G4ThreeVector(PosX,PosY,PosZ);
  pPathFinder->PrepareNewTrack(position,momDirection);
  G4TouchableHandle touchableParaW = pPathFinder-> CreateTouchableHandle(navigatorID);

  while(!touchableParaW->GetVolume())
  { position = G4ThreeVector(PosX + (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.,
                             PosY + (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.,
		             PosZ+ (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.);
  } // avoiding the sample edge

  if(!ParametersSetting::RadioAGrain) // non-radioactive grains, radioactive matrix
  {
   while(touchableParaW->GetVolume()->GetName()=="Grain") // random position selection until a point in the matrix is find
   {
    PosX = D_vox[0] *(1.*ParametersSetting::XposId[k_max]-N_Vox[0]/2.+0.5)+ offset[0]+D_DimDet[0]*(G4UniformRand()-0.5);
    PosY = D_vox[1] *(1.*ParametersSetting::YposId[k_max]-N_Vox[1]/2.+0.5)+ offset[1]+D_DimDet[1]*(G4UniformRand()-0.5);
    PosZ = D_vox[2] *(1.*ParametersSetting::ZposId[k_max]-N_Vox[2]/2.) + D_vox[2]*(G4UniformRand());
    position = G4ThreeVector(PosX,PosY,PosZ);
    pPathFinder->PrepareNewTrack(position,momDirection);
    touchableParaW = pPathFinder-> CreateTouchableHandle(navigatorID );
   }
  }
  else // radioactive grains, non-radioactive matrix
  {
   while(touchableParaW->GetVolume()->GetName()=="Detector") // random position selection until a point in the grains is find   
   {
    PosX = D_vox[0] *(1.*ParametersSetting::XposId[k_max]-N_Vox[0]/2.+0.5)+ offset[0]+D_DimDet[0]*(G4UniformRand()-0.5);
    PosY = D_vox[1] *(1.*ParametersSetting::YposId[k_max]-N_Vox[1]/2.+0.5)+ offset[1]+D_DimDet[1]*(G4UniformRand()-0.5);
    PosZ = D_vox[2] *(1.*ParametersSetting::ZposId[k_max]-N_Vox[2]/2.) + D_vox[2]*(G4UniformRand());
    position = G4ThreeVector(PosX,PosY,PosZ);
    pPathFinder->PrepareNewTrack(position,momDirection);
    touchableParaW = pPathFinder-> CreateTouchableHandle(navigatorID );
   }
  }
  InfMatrixE_Det[ParametersSetting::ZposId[k_max]] += Energy; // accumulate the energy in the detector map of energy deposited
 }

 //
 // accumulation of the energy of the particle emitted from a probe segment location
 //
 if(!ProbeNavigator)
 {
  G4String ProbWName = "ProbeWorld";
  G4VPhysicalVolume* pProbeW = G4TransportationManager::GetTransportationManager()->GetParallelWorld(ProbWName);
  ProbeNavigator =G4TransportationManager::GetTransportationManager()->GetNavigator(pProbeW );
 }
 ProbeWorldID = G4TransportationManager::GetTransportationManager()->ActivateNavigator(ProbeNavigator);
 position = G4ThreeVector(PosX,PosY,PosZ);
 pPathFinder->PrepareNewTrack(position,momDirection);
 G4TouchableHandle touchableProbW = pPathFinder-> CreateTouchableHandle(ProbeWorldID);
  if(touchableProbW->GetVolume()->GetName() == "Probe")
  {
   G4int ProbSegNum = touchableProbW->GetVolume()->GetCopyNo();
   Eemitted_Probe[ProbSegNum] += Energy;
  }

 Cpt +=1.; // incrementation of the number of particles emitted

 return Energy;
}
