//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "ParametersSetting.hh"

using namespace std;

//  static variables initialization

G4String ParametersSetting::ParamName;
G4String ParametersSetting::ResultName;

G4double ParametersSetting::NbPart; G4double ParametersSetting::clock;
G4int ParametersSetting::RadNat; G4double ParametersSetting::MoDir[3];
G4int ParametersSetting::RadEl; G4double ParametersSetting::CutValue;
G4int ParametersSetting::DetectorDef; G4int ParametersSetting::DetectorVox; 
G4int ParametersSetting::MatOfInterest; G4bool ParametersSetting::RadioAGrain;
G4int ParametersSetting::NbProbeSegment; G4double ParametersSetting::ProbeDiameter;
G4double ParametersSetting::ProbeOffSetXY[2];
G4int ParametersSetting::NbCompo;
G4int ParametersSetting::NbMat;

vector<G4String> ParametersSetting::CompName;
vector<G4double> ParametersSetting::CompDensity;
vector<G4int> ParametersSetting::Comp_NbAtom;
vector<G4String> ParametersSetting::Comp_ListAtom;
vector<G4int> ParametersSetting::Comp_ListAtNumber;

vector<G4String> ParametersSetting::MatName;
vector<G4double> ParametersSetting::MatDensity;
vector<G4double> ParametersSetting::MatWaterContent;
vector<G4int> ParametersSetting::Mat_NbComp;
vector<G4int> ParametersSetting::Mat_ListComponents;
vector<G4double> ParametersSetting::Mat_ListContents;

G4int ParametersSetting::NbVox_byAxis[3]; G4double ParametersSetting::DimVox[3]; G4double ParametersSetting::DimSample[3];
G4int ParametersSetting::NbVoxels;

vector<G4int> ParametersSetting::MaterialMap;
vector<G4double> ParametersSetting::RadContentMap;
vector<G4int> ParametersSetting::XposId;vector<G4int> ParametersSetting::YposId;vector<G4int> ParametersSetting::ZposId;

vector<G4double> ParametersSetting::Object_RadContentMap;
vector<G4int> ParametersSetting::Object_MaterialMap;
vector<G4int> ParametersSetting::Obj_XposId;vector<G4int> ParametersSetting::Obj_YposId;vector<G4int> ParametersSetting::Obj_ZposId;

vector<G4int> ParametersSetting::CompGrainId; vector<G4double> ParametersSetting::Graindensity;
vector<G4double> ParametersSetting::Compacity; vector<G4int> ParametersSetting::NbLineGranu;
map<G4int,vector<G4double> > ParametersSetting::Diameter; map<G4int,vector<G4double> > ParametersSetting::VolFraction;

G4int ParametersSetting::Object_voxelNumberXYZ[3];
G4int ParametersSetting::Object_NbVoxel;
G4double ParametersSetting::DimDet[3];
G4double ParametersSetting::DetOffSet[3];

// end of static variables initialization

ParametersSetting::ParametersSetting()
{;}

ParametersSetting::~ParametersSetting()
{;}

void ParametersSetting::SetStaticParamaters() // reading of the pilot file and static variables setting
{ 
 int Lim_int = std::numeric_limits<int>::max();

 // definition of the pilot macro file
 ParamName = "unknow";
 G4cout<<"pilot file name: "<<G4endl;
 G4cin>> ParamName; ParamName = "data/"+ParamName;
 // ParamName = "VoxSilex.txt"; ParamName = "data/"+ParamName;
 ifstream param;
 param.open(ParamName,ios::in);
 while(!param.is_open())
 {
  param.close();
  G4cout<< "impossible to open "<<ParamName<<G4endl
  <<"The available pilot files in the data folder are:"<<G4endl;
  DIR * rep = opendir("./data");
  if (rep != NULL)
  {
   struct dirent * ent;
   G4int fcolumn =0;
   while ((ent = readdir(rep)) != NULL)
   {
    G4String listPilot =  ent->d_name;
    G4String Pname = "data/" + listPilot;
    param.open(Pname,ios::in);
    if(param.is_open())
    {   
     if(listPilot != "spectra")G4cout<<ent->d_name<<"	";
     fcolumn ++;
     if(fcolumn%4==0)G4cout<<G4endl;
    }
    param.close();
   }      
   closedir(rep);
  }
  G4cout<<G4endl<<G4endl<<"pilot file name: "<<G4endl;
  G4cin>> ParamName; ParamName = "data/"+ParamName;
  param.open(ParamName,ios::in);
 }

 //
 // general parameters ///
 //

	// general data reading

 param>>ResultName;param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n'); // reading of the result file name and testing if already existing
 G4String RProbeName = ResultName + "_Probe";
 ifstream result;
 result.open(ParamName,ios::in);
 DIR * res = opendir("./results");
 if (res != NULL)
 {
  struct dirent * ent;
  while ((ent = readdir(res)) != NULL)
  {
   G4String listResult =  ent->d_name;
   if(listResult == RProbeName)
   {
    G4String fAnswer = "NULL";
    while((fAnswer != "yes")&&(fAnswer != "y"))
    {
     G4cout<<"a results file corresponding to - "<<ResultName<<" - already exists, the simulation while overwrite it"
     <<G4endl<<" Continue? (yes or no): "<<G4endl;G4cin>>fAnswer;
     if((fAnswer == "no")||(fAnswer == "n")){G4cout<<"closing of DosiVox"<<G4endl;abort();}
    }	 
   }	
  }
  closedir(res);
 }

 param>> NbPart >> clock; param.ignore(Lim_int,'\n');       // number of thousand emitted particles
 param>> RadNat>>MoDir[0]>>MoDir[1]>>MoDir[2]; param.ignore(Lim_int,'\n');       // particle emitted: 1 for alpha, 2 for beta, 3 for gamma
 param>> RadEl>>CutValue; param.ignore(Lim_int,'\n');        // element emitter : 1 for uranium series, 2 for thorium series, 3 for potassium
 param>> DetectorDef >> DetectorVox >> MatOfInterest >> RadioAGrain;param.ignore(Lim_int,'\n');   // index for detector definition
 param>>NbProbeSegment>>ProbeDiameter>>ProbeOffSetXY[0]>>ProbeOffSetXY[1];param.ignore(Lim_int,'\n');   // probe definition
 param>> NbCompo; param.ignore(Lim_int,'\n');      // number of new components defined
 param>> NbMat; param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');      // number of materials used

 // parameters adaptation
 NbPart*=1000.; 
 ProbeDiameter *= mm; ProbeOffSetXY[0] -=50 ;ProbeOffSetXY[1] -=50 ;
 ProbeOffSetXY[0] /=100. ;ProbeOffSetXY[1]  /=100. ;

 /// material parameters ///

 // vectors initialization

 CompName.reserve(0);CompDensity.reserve(NbCompo);Comp_NbAtom.reserve(NbCompo);
 Comp_ListAtom.reserve(0);Comp_ListAtNumber.reserve(0);
 MatName.reserve(0);Mat_ListComponents.reserve(0);Mat_ListContents.reserve(0);
 MatDensity.reserve(NbMat);MatWaterContent.reserve(NbMat);Mat_NbComp.reserve(NbMat);
 CompGrainId.reserve(NbMat);Graindensity.reserve(NbMat);Compacity.reserve(NbMat);NbLineGranu.reserve(NbMat);
 
 // new component data reading
 for(G4int i=0; i<NbCompo;i++)
 {
  G4int fint=0;G4String fstring; G4double fdouble=0.;
  param>>fint>>fstring>>fdouble>>fint;
  CompName.push_back(fstring);CompDensity[i]=fdouble;Comp_NbAtom[i]=fint;
  for(G4int j=0;j<Comp_NbAtom[i];j++)
  {
   param>>fstring>>fint;
   Comp_ListAtom.push_back(fstring);Comp_ListAtNumber.push_back(fint);
  }
  param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');
 }
 // material data reading
 for(G4int i=0; i<NbMat;i++)
 {
  G4int fint=0;G4String fstring; G4double fdouble=0.;
  param>>fint;param.ignore(Lim_int,'\n');
  if(fint!=i){G4cout<<"problem with material dial at material "<<i<<": material index is "<<fint<<G4endl;abort();}
  param>>fstring; MatName.push_back(fstring); param.ignore(Lim_int,'\n');// material name
  param>>fdouble; MatDensity[i]= fdouble; param.ignore(Lim_int,'\n'); // material densities
  param>>fdouble; MatWaterContent[i]= fdouble; param.ignore(Lim_int,'\n');
  param>>fint;	Mat_NbComp[i]= fint;param.ignore(Lim_int,'\n');

  for(G4int j=0;j<Mat_NbComp[i];j++)
  {
   param>>fint>>fdouble;
   Mat_ListComponents.push_back(fint);
   Mat_ListContents.push_back(fdouble);
  }
  param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');
   
  param>>fint>>fdouble; CompGrainId[i]=fint;Graindensity[i]=fdouble;
  param>>fdouble>>fint;Compacity[i]=fdouble/100.;NbLineGranu[i]=fint;
  param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');

  Diameter[i].reserve(NbLineGranu[i]);VolFraction[i].reserve(NbLineGranu[i]);
  for(G4int j=0;j<NbLineGranu[i];j++)
  {
   G4double Diam=0;G4double Fvol=0;
   param>>Diam>>Fvol;param.ignore(Lim_int,'\n');
   Diameter[i][j]=Diam;VolFraction[i][j]=Fvol/100.;
  }
  param.ignore(Lim_int,'\n');
 }
 
 //
 // 3D grid parameters
 //

 // 3D grid data reading
 param>>NbVox_byAxis[0]>>NbVox_byAxis[1]>>NbVox_byAxis[2]; param.ignore(Lim_int,'\n');      // voxels number by axis
 param>>DimVox[0]>>DimVox[1]>>DimVox[2]; param.ignore(Lim_int,'\n'); param.ignore(Lim_int,'\n');   // voxels sizes

 // parameters adaptation
 DimVox[0] *= mm;DimVox[1] *= mm;DimVox[2] *= mm;
 ProbeOffSetXY[0] *=DimVox[0] ;ProbeOffSetXY[1] *=DimVox[1] ;

 // geometry size calculation
 DimSample[0] = DimVox[0]*NbVox_byAxis[0];
 DimSample[1] = DimVox[1]*NbVox_byAxis[1];
 DimSample[2] = DimVox[2]*NbVox_byAxis[2];
 NbVoxels = NbVox_byAxis[0]*NbVox_byAxis[1]*NbVox_byAxis[2]; // total number of voxels of the main 3D grid

 //
 // voxels mapping parameters
 //

 // vectors initialization
 MaterialMap.reserve(NbVoxels);RadContentMap.reserve(NbVoxels);
 XposId.reserve(NbVoxels);YposId.reserve(NbVoxels);ZposId.reserve(NbVoxels);

 for(G4int i=0; i<NbVoxels; i++)
 {
  // voxels map navigation
  G4int Xi = i % NbVox_byAxis[0]; 
  G4int Yi = ((i-Xi)/NbVox_byAxis[0])%NbVox_byAxis[1]; 
  G4int Zi = (((((i-Xi)/NbVox_byAxis[0])-Yi)/NbVox_byAxis[1]))%NbVox_byAxis[2]; 
  if(Xi==0)param.ignore(Lim_int,'\n'); if((Yi==0)&&(Xi==0))param.ignore(Lim_int,'\n'); // lines passing to get the next map slice
  XposId[i]=Xi; YposId[i]=Yi; ZposId[i]=Zi; // recording of the voxels position
  param>>MaterialMap[i]; // material index reading, map reconstruction
 }
 param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');

 for(G4int i=0; i<NbVoxels; i++)
 {
  if(XposId[i]==0)param.ignore(Lim_int,'\n');
  if((YposId[i]==0)&&(XposId[i]==0))param.ignore(Lim_int,'\n');// lines passing to get the next map slice
  param>>RadContentMap[i]; // radio-element content reading, radioactive map reconstruction
 }

 //
 // Detectors definition and parameters
 //

 // Vertical probe detector only
 if(DetectorDef==0)
 { 
  DimDet[0] = min(DimVox[0],ProbeDiameter);
  DimDet[1] = min(DimVox[1],ProbeDiameter);
  DimDet[2] = min(DimVox[2],1.*(DimVox[2]*NbVox_byAxis[2]/NbProbeSegment));
 }

 // Sub-voxelised voxel detector
 if(DetectorDef==1)
 {
  param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');
  param>>Object_voxelNumberXYZ[0]>>Object_voxelNumberXYZ[1]>>Object_voxelNumberXYZ[2];param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');

  Object_NbVoxel = Object_voxelNumberXYZ[0]*Object_voxelNumberXYZ[1]*Object_voxelNumberXYZ[2]; // nb voxels in the sub-voxelised detector
  DimDet[0] = 1.*DimVox[0]/Object_voxelNumberXYZ[0];
  DimDet[1] = 1.*DimVox[1]/Object_voxelNumberXYZ[1];
  DimDet[2] = 1.*DimVox[2]/Object_voxelNumberXYZ[2]; // sub-voxel size

  Object_MaterialMap.reserve(Object_NbVoxel);Object_RadContentMap.reserve(Object_NbVoxel);
  Obj_XposId.reserve(Object_NbVoxel);Obj_YposId.reserve(Object_NbVoxel);Obj_ZposId.reserve(Object_NbVoxel); // vectors initialization

  for(G4int i=0; i<Object_NbVoxel; i++)
  { 
   G4int mat=0;
 
   Obj_XposId[i] = i % Object_voxelNumberXYZ[0];
   Obj_YposId[i] = ((i-Obj_XposId[i])/Object_voxelNumberXYZ[0])%Object_voxelNumberXYZ[1];
   Obj_ZposId[i] = ((((i-Obj_XposId[i])/Object_voxelNumberXYZ[0])-Obj_YposId[i])/Object_voxelNumberXYZ[1]);
			
   if(Obj_XposId[i]==0)param.ignore(Lim_int,'\n');
   if((Obj_YposId[i]==0)&&(Obj_XposId[i]==0))param.ignore(Lim_int,'\n');
   param>>mat;Object_MaterialMap[i]=mat; // material map data reading
  }
  param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');

  for(G4int i=0; i<Object_NbVoxel; i++)
  {
   G4double Rcont=0;
   if(Obj_XposId[i]==0)param.ignore(Lim_int,'\n');
   if((Obj_YposId[i]==0)&&(Obj_XposId[i]==0))param.ignore(Lim_int,'\n');
   param>>Rcont;Object_RadContentMap[i]=Rcont; // radioactive content map data reading
  }  
 }

 // single or successive random packings of grains detector
 if((DetectorDef==2)||(DetectorDef==3)) //
 {
  param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');param.ignore(Lim_int,'\n');
  param>>DimDet[0]>>DimDet[1]>>DimDet[2];param.ignore(Lim_int,'\n');
  DimDet[0]*=DimVox[0]/100.;DimDet[1]*=DimVox[1]/100.;DimDet[2]*=DimVox[2]/100.;
  param>>DetOffSet[0]>>DetOffSet[1]>>DetOffSet[2];param.ignore(Lim_int,'\n');
  DetOffSet[0] -= 50.;DetOffSet[1] -= 50.;DetOffSet[2] -= 50.;
  DetOffSet[0]*=DimVox[0]/100.;DetOffSet[1]*=DimVox[1]/100.;DetOffSet[2]*=DimVox[2]/100.;

  if(DetectorDef==3)
  {
   if(DimDet[2]>DimVox[2])DimDet[2]=DimVox[2];
   if((DimVox[2]/int(DimVox[2]/DimDet[2])-DimDet[2])<(DimDet[2]-DimVox[2]/(int(DimVox[2]/DimDet[2])+1)))
   DimDet[2]= DimVox[2]/int(DimVox[2]/DimDet[2]);
   else DimDet[2]=DimVox[2]/(int(DimVox[2]/DimDet[2])+1);
  }
 }
param.close();

}
