//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "PhysicsList.hh"
#include "G4EmLivermorePhysics.hh"
#include "G4EmStandardPhysics.hh"
#include "G4EmExtraPhysics.hh"
#include "G4DecayPhysics.hh"
#include "G4EmPenelopePhysics.hh"
#include "G4EmProcessOptions.hh"
#include "G4ParticleDefinition.hh"
#include "G4ParticleTable.hh"
#include "G4PhysicsListHelper.hh"
#include "G4ProcessManager.hh"
#include "G4ProcessVector.hh"
#include "G4PhysicalConstants.hh"

PhysicsList::PhysicsList(G4String& parWorldName): G4VModularPhysicsList()
,pWorldName(parWorldName)
{ 
 defaultCutValue = ParametersSetting::CutValue*mm;
 G4int ver = 0;
 SetVerboseLevel(ver);
  
 emPhysicsList = new G4EmPenelopePhysics(0);
 //emPhysicsList = new G4EmStandardPhysics();
 //emPhysicsList = new G4EmLivermorePhysics();
}

void PhysicsList::ConstructParticle()
{
 emPhysicsList->ConstructParticle();
}

PhysicsList::~PhysicsList()
{delete emPhysicsList;}

void PhysicsList::ConstructProcess()
{
 G4cout<<"definition of the simulation physics"<<G4endl<<G4endl;

 AddTransportation();  

 AddDetectorWorldProcess(); 

 emPhysicsList->ConstructProcess();
}

void PhysicsList::AddDetectorWorldProcess()
{
 // Add parallel world process
 G4ParallelWorldProcess* theParallelWorldProcess= new G4ParallelWorldProcess("paraWorldProc");
 theParallelWorldProcess->SetParallelWorld(pWorldName);
 theParallelWorldProcess->SetLayeredMaterialFlag();

 G4ParallelWorldProcess* theProbeWorldProcess= new G4ParallelWorldProcess("ProbeWorldProc");
 theProbeWorldProcess->SetParallelWorld("ProbeWorld");

 theParticleIterator->reset();
 while( (*theParticleIterator)() )
 {
  G4ParticleDefinition* particle = theParticleIterator->value();
  G4ProcessManager* pmanager = particle->GetProcessManager();
  pmanager->AddProcess(theParallelWorldProcess);
  pmanager->AddProcess(theProbeWorldProcess); 
  if(theParallelWorldProcess->IsAtRestRequired(particle))pmanager->SetProcessOrdering(theParallelWorldProcess, idxAtRest,9999);
  if(theProbeWorldProcess->IsAtRestRequired(particle))pmanager->SetProcessOrdering(theProbeWorldProcess, idxAtRest,9999);
  pmanager->SetProcessOrdering(theParallelWorldProcess, idxAlongStep, 1);
  pmanager->SetProcessOrdering(theProbeWorldProcess, idxAlongStep, 1);
  pmanager->SetProcessOrdering(theParallelWorldProcess, idxPostStep,9999);
  pmanager->SetProcessOrdering(theProbeWorldProcess, idxPostStep,9999);
 }
}

 void PhysicsList::SetCuts()
{
 SetCutsWithDefault(); 
} 
