//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "PrimaryGeneratorAction.hh"

using namespace std;

PrimaryGeneratorAction::PrimaryGeneratorAction(RunAction* run):myRunAct(run)
{
 // variables initialization
 Cpt = 0;      // counter resetting
 end = false;
 Period = ParametersSetting::clock;
 particleName = "NULL";

 G4int n_particle = 1;
 particleGun  = new G4ParticleGun(n_particle);
  
 // default particle kinematic
 G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
 particle = particleTable->FindParticle(particleName="geantino");
 particleGun->SetParticleDefinition(particle); 
 particleGun->SetParticlePosition(G4ThreeVector(0,0,0));  
 particleGun->SetParticleMomentumDirection(G4RandomDirection());  
 particleGun->SetParticleEnergy(0.*keV);

}

PrimaryGeneratorAction::~PrimaryGeneratorAction()
{ 
 delete particleGun;
}


void PrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{ 
 static G4ParticleTable* particleTable;
 if(!particleTable) particleTable = G4ParticleTable::GetParticleTable();

 G4ThreeVector position(0.,0.,0.);
 Energy=0;

 if(Cpt == 0)
 {
  NbEvent = ParametersSetting::NbPart;
  EmParam.ConstructSpectrum(ParametersSetting::RadEl,ParametersSetting::RadNat);
  G4cout<<G4endl;
  if(ParametersSetting::RadEl==1)G4cout<<"238U & 235U series";
  if(ParametersSetting::RadEl==2)G4cout<<"232Th series";
  if(ParametersSetting::RadEl==3)G4cout<<"K";
  if(ParametersSetting::RadEl==0)G4cout<<"User Defined";
  if((ParametersSetting::RadEl>3)||(ParametersSetting::RadEl<0)){G4cout<<"unknown radioactive element/series index"<<G4endl; abort();}

  if(ParametersSetting::RadNat==1)G4cout<<" alpha spectrum"<<G4endl;
  if(ParametersSetting::RadNat==2)G4cout<<" beta spectrum"<<G4endl;
  if(ParametersSetting::RadNat==3)G4cout<<" gamma spectrum"<<G4endl;
  if((ParametersSetting::RadNat>3)||(ParametersSetting::RadNat<1)){G4cout<<" unknown particle index"<<G4endl; abort();}

  G4cout<<"number of particles to generate: "<<ParametersSetting::NbPart<<G4endl
        <<"beginning of the simulation"<<G4endl;
 }

 if(Cpt==1)myRunAct->InitAtBeginOfRun(false);

 G4double count =(Cpt*100/NbEvent/Period-int(Cpt*100/NbEvent/Period));
 if((count==0)&&(!end))
 {
  G4cout<<'\r'<<"\033[K"<<"progression: "<<Cpt<<"/"<<NbEvent<<" "<<Cpt*100./NbEvent<<"%"; 
  G4cout.flush() ;           // progression display
 }

 if(Cpt >= NbEvent)
 { 
  if(!end)
  { 
   end = true;
   myRunAct->WriteAtEndOfRun(true);
  }
  else myRunAct->WriteAtEndOfRun(false);
  G4RunManager::GetRunManager()->AbortRun(true); // end of the simulation when the requested number of particles emitted is reach
 }

 //
 // particle setting and emission
 //

 if(!end)
 {
  if(ParametersSetting::RadNat == 1)particle = particleTable->FindParticle(particleName="alpha");
  if(ParametersSetting::RadNat == 2)particle = particleTable->FindParticle(particleName="e-");
  if(ParametersSetting::RadNat == 3)particle = particleTable->FindParticle(particleName="gamma");

  Energy = EmParam.CalculateOneEmission_GetEnergy(); // calculate energy and position of the emitted particle
  position = G4ThreeVector(EmParam.GetPositionX(),EmParam.GetPositionY(),EmParam.GetPositionZ());


  // Set particle gun   
  particleGun->SetParticleEnergy(Energy); 
  particleGun->SetParticleDefinition(particle);
  particleGun->SetParticlePosition(position);
  if((ParametersSetting::MoDir[0]==0)&&(ParametersSetting::MoDir[1]==0)&&(ParametersSetting::MoDir[2]==0)) // particle direction
  particleGun->SetParticleMomentumDirection(G4RandomDirection());
  else particleGun->SetParticleMomentumDirection(G4ThreeVector(ParametersSetting::MoDir[0],
                                                               ParametersSetting::MoDir[1],
                                                               ParametersSetting::MoDir[2]));
   particleGun->GeneratePrimaryVertex(anEvent); // emission of the particle
   myRunAct->GetInfiniteMatrixEnergy(Energy); // accumulating the energy of emitted particles
   Cpt += 1.; // incrementing the number of particles emitted
 }
    
}

