//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "ProbeParallelWorld.hh"


ProbeParallelWorld::ProbeParallelWorld(G4String& parallelWorldName)
:G4VUserParallelWorld(parallelWorldName),constructed(false)
{ 
 // initialisation des variables
}

ProbeParallelWorld::~ProbeParallelWorld()
{;}

void ProbeParallelWorld::Construct()
{
 if(constructed) return;
 constructed = true;

 //
 // World
 //
 G4VPhysicalVolume* World_P = GetWorld();
 G4LogicalVolume* worldLog_P = World_P->GetLogicalVolume();

 //
 // mother of parallel world volumes
 //
 G4float ProbWx = 1.5*ParametersSetting::DimSample[0];
 G4float ProbWy = 1.5*ParametersSetting::DimSample[1];
 G4float ProbWz = 1.5*ParametersSetting::DimSample[2]; // world dimensions

 ProbeMomSol = new G4Box("ProbeParaMom",ProbWx/2.,ProbWy/2.,ProbWz/2.);
 ProbeMomLog = new G4LogicalVolume(ProbeMomSol,0,"ProbeParaMom",0,0,0);
 new G4PVPlacement(0,G4ThreeVector(0.,0.,0.),ProbeMomLog,"ProbeParaMom",worldLog_P,false,0);

 G4Box*	MirrorSol = new G4Box("MirrorBox",ParametersSetting::DimSample[0]/2.*(1.00001),
                                           ParametersSetting::DimSample[1]/2.*(1.00001),
                                           ParametersSetting::DimSample[2]/2.*(1.00001));
 G4LogicalVolume* MirrorLog = new G4LogicalVolume(MirrorSol,0,"MirrorBox",0,0,0);
 new G4PVPlacement(0,G4ThreeVector(0.,0.,0.),MirrorLog,"MirrorBox",ProbeMomLog,false,0);

 G4VisAttributes* MirrorVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
 MirrorVis->SetForceSolid(false);
 MirrorVis->SetVisibility(false);
 MirrorLog->SetVisAttributes(MirrorVis);

 G4double X_probe = (ParametersSetting::XposId[ParametersSetting::DetectorVox]+0.5)
		    *ParametersSetting::DimVox[0]+ParametersSetting::ProbeOffSetXY[0]-ParametersSetting::DimSample[0]/2.;
 G4double Y_probe = (ParametersSetting::YposId[ParametersSetting::DetectorVox]+0.5)
		    *ParametersSetting::DimVox[1]+ParametersSetting::ProbeOffSetXY[1]-ParametersSetting::DimSample[1]/2.;

 if((fabs(X_probe)+ParametersSetting::ProbeDiameter/2.>ProbWx/2.)||(fabs(Y_probe)+ParametersSetting::ProbeDiameter/2.>ProbWy/2.))
 {
  G4cout<<" error : probe is outside the geometry "<<G4endl;
  abort();
 }
 
 // probe shell creation
 G4Tubs* probe_shell_Sol = new G4Tubs("Probe_shell",0.,ParametersSetting::ProbeDiameter/2.,ParametersSetting::DimSample[2]/2.,0.,2*M_PI);
 G4LogicalVolume* probe_shell_Log = new G4LogicalVolume(probe_shell_Sol,0,"Probe_shell",0,0,0);
 new G4PVPlacement(0,G4ThreeVector(X_probe,Y_probe,0.),probe_shell_Log ,"Probe_shell",MirrorLog,false,0);


 G4VisAttributes* ProbeVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
 ProbeVis->SetForceSolid(false);
 ProbeVis->SetVisibility(false);
 probe_shell_Log->SetVisAttributes(ProbeVis);

 // probe segments creation
 G4double ProbeSegDim = ParametersSetting::DimSample[2]/ParametersSetting::NbProbeSegment;
 G4Tubs* probe_Sol = new G4Tubs("Probe",0.,ParametersSetting::ProbeDiameter/2.,ProbeSegDim/2.,0.,2*M_PI);
 G4LogicalVolume* probe_Log = new G4LogicalVolume(probe_Sol,0,"Probe",0,0,0);

 for(G4int i=0;i<ParametersSetting::NbProbeSegment;i++)
 {
  new G4PVPlacement(0,G4ThreeVector(0.,0.,ProbeSegDim*(i+0.5)-ParametersSetting::DimSample[2]/2.),
                    probe_Log ,"Probe",probe_shell_Log,false,i);

 }

 G4VisAttributes* ProbeSegVis= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
 ProbeSegVis->SetForceSolid(false);
 ProbeSegVis->SetVisibility(true);
 probe_Log->SetVisAttributes(ProbeSegVis);

 G4cout<<"successful creation of the probe detector"<<G4endl<<G4endl;

}

