//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "RPOrbsParam.hh"

using namespace std;

RPOrbsParam::RPOrbsParam(G4int MatIndex)
{
 NbPieceGrain =0;
 G4String GrainMatName = ParametersSetting::MatName[MatIndex]+"_grain";
 MatGrain = G4Material::GetMaterial(GrainMatName);

 //
 // creating a random packing of grains with the data of the material "MatIndex"
 //
 G4int NbDet=0;
 while(NbDet<1) // new random packings are calculated until at least one grain does not overlap the packing box boundaries
 {
  RandPack.CalcRandomPacking(MatIndex);
  NbDet=RandPack.GetWholeGrainNumber();
 }
 // getting the data of the calculated packing
 NbPieceGrain = RandPack.GetGrainPiecesNumber();
 x.reserve(NbPieceGrain);y.reserve(NbPieceGrain);z.reserve(NbPieceGrain);fDiametre.reserve(NbPieceGrain);
 G4int colorIt =0;
 for(G4int i =0;i<NbPieceGrain;i++)
 {
  fDiametre[i]=RandPack.GetGrainDiameter(i);
  if(!RandomPacking::IsValidDetector[i])fDiametre[i]*=-1.;
  x[i]=RandPack.GetGrainPosX(i);
  y[i]=RandPack.GetGrainPosY(i);
  z[i]=RandPack.GetGrainPosZ(i);
 
  map<G4double, G4VisAttributes*>::iterator it = GrainColor.find(fDiametre[i]);
  if(it == GrainColor.end())
  {  
   G4int Xcolor = (colorIt+1)%2;
   G4int Ycolor = (((colorIt+1)-Xcolor)/2)%2;
   G4int Zcolor = (((((colorIt+1)-Xcolor)/2)-Ycolor)/2)%2;
   if(fDiametre[i]>0.)
   {
    GrainColor[fDiametre[i]] = new G4VisAttributes(G4Colour(Xcolor,Ycolor,Zcolor));
    GrainColor[-fDiametre[i]] = new G4VisAttributes(G4Colour(Xcolor,Ycolor,Zcolor,0.5));
   }
   else
   {
    GrainColor[-fDiametre[i]] = new G4VisAttributes(G4Colour(Xcolor,Ycolor,Zcolor));
    GrainColor[fDiametre[i]] = new G4VisAttributes(G4Colour(Xcolor,Ycolor,Zcolor,0.5));
   }
    colorIt ++;
   GrainColor[fDiametre[i]]->SetForceSolid(true);
   GrainColor[fDiametre[i]]->SetVisibility(true);
   GrainColor[-fDiametre[i]]->SetForceSolid(true);
   GrainColor[-fDiametre[i]]->SetVisibility(true);
   
  }
 }

 voxel = new G4Box("cube",ParametersSetting::DimDet[0]/2,ParametersSetting::DimDet[1]/2,ParametersSetting::DimDet[2]/2);

}

RPOrbsParam::~RPOrbsParam()
{;}

void RPOrbsParam::ComputeTransformation(const G4int copyNo,G4VPhysicalVolume *physVol) const
{
 G4ThreeVector Pos(x[copyNo],y[copyNo],z[copyNo]);
 physVol->SetTranslation(Pos); 
}// grain position parameterization

G4Material* RPOrbsParam::ComputeMaterial(const G4int copyNo,G4VPhysicalVolume*PVol,const G4VTouchable* )
{
 it_color = GrainColor.find(fDiametre[copyNo]);
 PVol->GetLogicalVolume()->SetVisAttributes(it_color->second);
 return MatGrain;
} // grain material parameterization

G4VSolid* RPOrbsParam::ComputeSolid(const G4int copyNo, G4VPhysicalVolume* )
{
 sol =0;
 it_grain = fSolMap.find(copyNo);
 if(it_grain != fSolMap.end())sol = it_grain->second; // return the grain if already created
 else
 {  
  G4Orb* orb = new G4Orb("grain",fabs(fDiametre[copyNo])/2.);
	
  if((fabs(x[copyNo])+fabs(fDiametre[copyNo])/2.<ParametersSetting::DimDet[0]/2.)
    &&(fabs(y[copyNo])+fabs(fDiametre[copyNo])/2.<ParametersSetting::DimDet[1]/2.)
    &&(fabs(z[copyNo])+fabs(fDiametre[copyNo])/2.<ParametersSetting::DimDet[2]/2.))
  sol = orb; // if the grain does not overlap the packing box boundaries, a spherical shape is created
  else
  {
   G4ThreeVector trans(-x[copyNo],-y[copyNo],-z[copyNo]);
   G4IntersectionSolid* inter = new G4IntersectionSolid("grain_bord",orb,voxel,0,trans);
   sol = inter;
  }  // if the grain overlaps the packing box boundaries, a boolean solid shape is created by the intersection of the grain and the box
  fSolMap[copyNo] = sol; // the created grain shape is registered 
 }
 return sol;

} // grain size and shape parameterization


