//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "RandomPacking.hh"

using namespace std;

vector<G4bool> RandomPacking::IsValidDetector;

 RandomPacking::RandomPacking()
 {;}
 
  RandomPacking::~RandomPacking()
  { ;}

void  RandomPacking::CalcRandomPacking(G4int MatId)
{
 NbGrain=0;NbPieceGrain=0;NbWholeGrain=0;densi=0;
 X=0;Y=0;Z=0;diam=0;prop=0;VolRelatif=0;
 x.clear();y.clear();z.clear();fDiametre.clear();
 xFinal.clear();yFinal.clear();zFinal.clear();fDiametreFinal.clear();

 G4double TotalVolGrain = ParametersSetting::DimDet[0]*ParametersSetting::DimDet[1]*ParametersSetting::DimDet[2]*ParametersSetting::Compacity[MatId];
 vector<G4int> NbGrainByDiam; NbGrainByDiam.reserve(ParametersSetting::NbLineGranu[MatId]);

 G4cout<<G4endl<<"Calculating grain packing in material "<<MatId<<" ("<<ParametersSetting::MatName[MatId]<<")"<<G4endl;

 //
 // number of grains of each size in the packing
 //  
 for(G4int i=0;i<ParametersSetting::NbLineGranu[MatId];i++)
 {
  G4double NGrain = ParametersSetting::VolFraction[MatId][i]*TotalVolGrain/(4./3.*M_PI*pow(ParametersSetting::Diameter[MatId][i]/2.,3.));
  G4int nGrain = int(NGrain);       
  if(NGrain-nGrain>0.5)nGrain++;
  NbGrainByDiam[i]=nGrain;
  NbGrain += nGrain;

  G4cout<<"Diameter: "<<ParametersSetting::Diameter[MatId][i]<<" mm, nb of grains: "<<NGrain<<" -> corrected: "<<NbGrainByDiam[i]<<G4endl;
 }

 G4cout<<"total number of grains in material "<<MatId<<" ("<<ParametersSetting::MatName[MatId]<<")"<<" : "
 <<NbGrain<<", compacity: "<<ParametersSetting::Compacity[MatId]*100.<<" %"<<G4endl;

 if(NbGrain==0){G4cout<<"ERROR: Number of grain < 1 in material "<<MatId<<" "<<" ("<<ParametersSetting::MatName[MatId]
                      <<"): impossible to use as detector"<<G4endl
		      <<"please create a larger volume for packing or suppress the grains in this material"<<G4endl;abort();}


 x.reserve(NbGrain);y.reserve(NbGrain);z.reserve(NbGrain);fDiametre.reserve(NbGrain);

 //
 // setting the diameter of each grain 
 //    
 G4int k=0;
 for(G4int i=0;i<ParametersSetting::NbLineGranu[MatId];i++)
 {
  for(G4int j=0;j<NbGrainByDiam[i];j++)
  {
   fDiametre[k]=ParametersSetting::Diameter[MatId][i]*mm;
   k++; 
  }
 }

 //
 // random positionning of the grains
 //
 G4double D_cart=0;
 G4int grainID=0;
 G4bool recou=true;
 G4double M_diam = 0;
    
 if(ParametersSetting::Compacity[MatId]*100.<=35)M_diam = 1.; // random positionning with the final size of the grains
 else M_diam = 0.85; // random positionning with a reduced size of grains

 for(G4int i=0; i<NbGrain; i++)
 {
  recou = true;
  while(recou)
  {
   x[i] = (G4UniformRand()-0.5)*ParametersSetting::DimDet[0]; // random setting of the position in the packing box
   y[i] = (G4UniformRand()-0.5)*ParametersSetting::DimDet[1];
   z[i] = (G4UniformRand()-0.5)*ParametersSetting::DimDet[2];

   recou = false;
   grainID=0;
   for(grainID=0; grainID<i; grainID++) // checking for overlapping
   {
    for(G4int q=0;q<27;q++) // checking for overlapping with recurent boundaries
    {
     X = q%3;
     Y = ((q-X)/3)%3;
     Z = ((((q-X)/3)-Y)/3)%3;
     X--;Y--;Z--;

     D_cart= pow(pow(x[i]-(x[grainID]+X*ParametersSetting::DimDet[0]),2.)
                 + pow(y[i]-(y[grainID]+Y*ParametersSetting::DimDet[1]),2.)
                 + pow(z[i]-(z[grainID]+Z*ParametersSetting::DimDet[2]),2.),0.5);

     if((i != grainID)&&(D_cart< (fDiametre[i]+fDiametre[grainID])/2.*M_diam) ) // an overlaps is detected
     {
      recou = true; 
      break; 
     }
    }
    if(recou)break;
   }
  }
  if((int((i+1)*100/NbGrain)%1==0)&&(int((i+1)*100/NbGrain)>int(i*100/NbGrain)))
  {
   G4cout<<'\r'<<"\033[K"<<"grains random positioning: "<<(i+1)*100/NbGrain<<"%, grain size = "<<M_diam*100<<"%"; 
   G4cout.flush() ; // displaying the grain positionning progression
  } 
 }
 G4cout<<G4endl;

 //
 // grain compaction and repulsion calculation
 //
 G4int P_diam=int(M_diam*100);
 G4double Di,Dg,D_grain,dx,dy,dz;
 G4bool rePos = false;
 X=0;Y=0;Z=0;
 G4int Nbcoll = 0;

 if(P_diam<100)
 {
  G4cout<<"Increasing the grain size to reach the requested compacity:"<<G4endl;
  for(P_diam=int(M_diam*100);P_diam<=100;P_diam ++)
  {
   // displaying the grain compaction progression
   G4cout<<'\r'<<"\033[K"<<"Grain size: "<<(P_diam-1)<<"% "<<"number of repulsions: "<<Nbcoll; 
   G4cout.flush() ;
   Nbcoll = 0;

   for(G4int i=0; i<NbGrain; i++)
   {
    Di = fDiametre[i]*pow(P_diam/100.,1.);
    grainID = i+1; // the non-overlapping of the grains from 0 to i is already checked
    for( grainID=i+1; grainID < NbGrain; grainID++) // checking for overlapping grains
    {
     rePos = false;
     Dg = fDiametre[grainID]*pow(P_diam/100.,1.);
     for(G4int q=0;q<27;q++) // checking for overlapping with recurent boundaries
     {
      X = q%3;
      Y = ((q-X)/3)%3;
      Z = ((((q-X)/3)-Y)/3)%3;
      X--;Y--;Z--;

      // distance between the centers of grain i and g
      D_cart= pow(pow(x[i]-(x[grainID]+X*ParametersSetting::DimDet[0]),2.) 
                 +pow(y[i]-(y[grainID]+Y*ParametersSetting::DimDet[1]),2.) 
                 +pow(z[i]-(z[grainID]+Z*ParametersSetting::DimDet[2]),2.),0.5);
      D_grain = D_cart-(Di+Dg)/2.; // minimal distance between the two grains surfaces

      if((i != grainID)&&(D_grain < 0))
      {
       // distance for separating the grains
       dx = (x[i]-(x[grainID]+X*ParametersSetting::DimDet[0]))*(-D_grain/D_cart);
       dy = (y[i]-(y[grainID]+Y*ParametersSetting::DimDet[1]))*(-D_grain/D_cart);
       dz = (z[i]-(z[grainID]+Z*ParametersSetting::DimDet[2]))*(-D_grain/D_cart);

       // grain i recoil
       x[i] += dx*pow(Dg,3.)/(pow(Di,3.)+pow(Dg,3.))+dx/fabs(dx)*0.005*micrometer;
       y[i] += dy*pow(Dg,3.)/(pow(Di,3.)+pow(Dg,3.))+dy/fabs(dy)*0.005*micrometer;
       z[i] += dz*pow(Dg,3.)/(pow(Di,3.)+pow(Dg,3.))+dz/fabs(dz)*0.005*micrometer;

       // grain g recoil
       x[grainID] -= dx*pow(Di,3.)/(pow(Di,3.)+pow(Dg,3.))+dx/fabs(dx)*0.005*micrometer;
       y[grainID] -= dy*pow(Di,3.)/(pow(Di,3.)+pow(Dg,3.))+dy/fabs(dy)*0.005*micrometer;
       z[grainID] -= dz*pow(Di,3.)/(pow(Di,3.)+pow(Dg,3.))+dz/fabs(dz)*0.005*micrometer;

       rePos = true;
       }
       // re-positionnig the grain if it gets out of the box
       if(fabs(x[i])>ParametersSetting::DimDet[0]/2.){x[i]-= (x[i]/fabs(x[i]))*ParametersSetting::DimDet[0];}
       if(fabs(y[i])>ParametersSetting::DimDet[1]/2.){y[i]-= (y[i]/fabs(y[i]))*ParametersSetting::DimDet[1];}
       if(fabs(z[i])>ParametersSetting::DimDet[2]/2.){z[i]-= (z[i]/fabs(z[i]))*ParametersSetting::DimDet[2];}
       if(fabs(x[grainID])>ParametersSetting::DimDet[0]/2.){x[grainID]-= (x[grainID]/fabs(x[grainID]))*ParametersSetting::DimDet[0];}
       if(fabs(y[grainID])>ParametersSetting::DimDet[1]/2.){y[grainID]-= (y[grainID]/fabs(y[grainID]))*ParametersSetting::DimDet[1];}
       if(fabs(z[grainID])>ParametersSetting::DimDet[2]/2.){z[grainID]-= (z[grainID]/fabs(z[grainID]))*ParametersSetting::DimDet[2];}

       if(rePos){Nbcoll++;break;} // if a repulsion has been calculated, the loop is reinitialized
      }
      if(rePos){i=-1;break;}
     }
    }
  }
  G4cout<<'\r'<<"\033[K"<<"Grains size: "<<(P_diam-1)<<"% "<<"number of repulsions: "<<Nbcoll; 
  G4cout.flush() ; // displaying the grain compaction progression
 }

 G4cout<<G4endl<<"Grains packing for material "<<MatId<<" "<<" ("<<ParametersSetting::MatName[MatId]<<") calculated"<<G4endl
       <<"registration of the grains position and geometry"<<G4endl;

 //
 // calculation of the number of pieces of grains
 //
 NbPieceGrain = 0;
 NbWholeGrain =0;

 for(G4int i=0;i<NbGrain;i++)
 {
  for(G4int q=0;q<27;q++) // checking the piece of grains from recurrent boundaries
  {
   X = q%3;
   Y = ((q-X)/3)%3;
   Z = ((((q-X)/3)-Y)/3)%3;
   X--;Y--;Z--;

   if((fabs(x[i]-X*ParametersSetting::DimDet[0])<ParametersSetting::DimDet[0]/2.+fDiametre[i]/2.)
     &&(fabs(y[i]-Y*ParametersSetting::DimDet[1])<ParametersSetting::DimDet[1]/2.+fDiametre[i]/2.)
     &&(fabs(z[i]-Z*ParametersSetting::DimDet[2])<ParametersSetting::DimDet[2]/2.+fDiametre[i]/2.))
   NbPieceGrain ++; // counting the number of pieces of grains in the packing box
  }
  if((fabs(x[i])<ParametersSetting::DimDet[0]/2.-fDiametre[i]/2.)
    &&(fabs(y[i])<ParametersSetting::DimDet[1]/2.-fDiametre[i]/2.)
    &&(fabs(z[i])<ParametersSetting::DimDet[2]/2.-fDiametre[i]/2.) )
  NbWholeGrain++; // counting the number of whole grains in the packing box (non-overlapping with the box boundaries)
 }

 IsValidDetector.clear();
 IsValidDetector.reserve(NbPieceGrain);
 fDiametreFinal.reserve(NbPieceGrain);
 xFinal.reserve(NbPieceGrain);
 yFinal.reserve(NbPieceGrain);
 zFinal.reserve(NbPieceGrain); // vector initialization

 //
 // registration of the pieces of grains coordinates
 //
 G4int PieceOfGrain=0;

 for(G4int i=0;i<NbGrain;i++)
 {
  for(G4int q=0;q<27;q++) // checking the pieces of grains from recurrent boundaries
  {
   X = q%3;
   Y = ((q-X)/3)%3;
   Z = ((((q-X)/3)-Y)/3)%3;
   X--;Y--;Z--;

   if((fabs(x[i]-X*ParametersSetting::DimDet[0])<ParametersSetting::DimDet[0]/2.+fDiametre[i]/2.)
     &&(fabs(y[i]-Y*ParametersSetting::DimDet[1])<ParametersSetting::DimDet[1]/2.+fDiametre[i]/2.)
     &&(fabs(z[i]-Z*ParametersSetting::DimDet[2])<ParametersSetting::DimDet[2]/2.+fDiametre[i]/2.))
   {
    fDiametreFinal[PieceOfGrain] = fDiametre[i];
    xFinal[PieceOfGrain]=x[i]-X*ParametersSetting::DimDet[0];
    yFinal[PieceOfGrain]=y[i]-Y*ParametersSetting::DimDet[1];
    zFinal[PieceOfGrain]=z[i]-Z*ParametersSetting::DimDet[2];

    IsValidDetector[PieceOfGrain]=0;
    if((fabs(x[i])<ParametersSetting::DimDet[0]/2.-fDiametre[i]/2.)
      &&(fabs(y[i])<ParametersSetting::DimDet[1]/2.-fDiametre[i]/2.)
      &&(fabs(z[i])<ParametersSetting::DimDet[2]/2.-fDiametre[i]/2.) )
    IsValidDetector[PieceOfGrain]=1; // if the grain does not overlap the packing box boundaries, it is a valid detector for dosimetry

    PieceOfGrain ++;
   } 
  }
 }

x.clear();y.clear();z.clear();fDiametre.clear();

 G4cout<<"Number of grains validated as detector in the material - "<<ParametersSetting::MatName[MatId]
 <<" - : "<<NbWholeGrain<<G4endl<<G4endl;

}


