//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "RunAction.hh"
#include "G4Run.hh"

using namespace std;

RunAction::RunAction()
{
 InfMatE = 0;
 Eloss = 0.;
 NbPartkilled=0;
 begin = true;
 end =false;

 //
 // detector sheets initialization
 //
 NbSegProb = ParametersSetting::NbProbeSegment; // cylindrical probe detector
 ProbeE.reserve(ParametersSetting::NbProbeSegment);// reserve the size of the vector
 for(G4int i=0;i<ParametersSetting::NbProbeSegment;i++)ProbeE[i]=0.;// vector initialization

 if(ParametersSetting::DetectorDef==1) // sub-voxelised voxel (detector 1)
 {
  Det1E.reserve(ParametersSetting::Object_NbVoxel);// reserve the size of the vector
  for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)Det1E[i]=0; // vector initialization
 }

 if(ParametersSetting::DetectorDef==2) // single grain packing (detector 2)
 {
  map<G4int,G4double> E;
  for(std::map<G4int,G4double>::iterator
      it=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].begin();
      it!=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].end();++it)
  E[it->first]=0.; // reserve the size of the vector
        
  DetE_2_3.push_back(E);
  for(std::map<G4int,G4double>::iterator
      it=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].begin();
      it!=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].end();++it)
  DetE_2_3[0][it->first] =0.; // vector initialization
 }

 if(ParametersSetting::DetectorDef==3) // successive grain packings (detector 3)
 {  
  G4int nZ =0;
  for(G4int i=0;i<ParametersSetting::NbVoxels;i++)
  {
   if((ParametersSetting::XposId[i]==ParametersSetting::XposId[ParametersSetting::DetectorVox])
     &&(ParametersSetting::YposId[i]==ParametersSetting::YposId[ParametersSetting::DetectorVox]))
   {
    for(G4int j=0;j< int(ParametersSetting::DimVox[2]/ParametersSetting::DimDet[2]);j++)
    {
     map<G4int,G4double> E;
     for(std::map<G4int,G4double>::iterator
         it=DetectorParallelWorld::DiamVoxMap[i].begin();
         it!=DetectorParallelWorld::DiamVoxMap[i].end();++it)
     E[it->first]=0.;// creation of a map with the size for recording the dose in the packing box nZ
                    
     DetE_2_3.push_back(E);// reserve the size of the map in the vector DetE_2_3

     for(std::map<G4int,G4double>::iterator
     it=DetectorParallelWorld::DiamVoxMap[i].begin();
     it!=DetectorParallelWorld::DiamVoxMap[i].end();++it)
     DetE_2_3[nZ][it->first] =0.; // map initialization
			
     nZ++; // incrementing the packing box index
    }
   }
  }
 }

}

RunAction::~RunAction()
{;}

void RunAction::BeginOfRunAction(const G4Run* /*aRun*/)
{ 
 if(begin)
 {
  InfMatE = 0;
  Eloss = 0.;
  NbPartkilled=0;
  end =false;
  ProbeE.clear();
  Det1E.clear();
  DetE_2_3.clear();

  //
  // detector sheets re-initialization (usefull when running successive simulations with the visualization activated)
  //
  NbSegProb = ParametersSetting::NbProbeSegment; // cylindrical probe detector
  ProbeE.reserve(ParametersSetting::NbProbeSegment);// reserve the size of the vector
  for(G4int i=0;i<ParametersSetting::NbProbeSegment;i++)ProbeE[i]=0.;// vector initialization

  if(ParametersSetting::DetectorDef==1) // sub-voxelised voxel (detector 1)
  {
   Det1E.reserve(ParametersSetting::Object_NbVoxel);// reserve the size of the vector
   for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)Det1E[i]=0; // vector initialization
  }

  if(ParametersSetting::DetectorDef==2) // single grain packing (detector 2)
  {
   map<G4int,G4double> E;
   for(std::map<G4int,G4double>::iterator
       it=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].begin();
       it!=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].end();++it)
   E[it->first]=0.; // reserve the size of the vector
        
   DetE_2_3.push_back(E);
   for(std::map<G4int,G4double>::iterator
       it=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].begin();
       it!=DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox].end();++it)
   DetE_2_3[0][it->first] =0.; // vector initialization
  }

  if(ParametersSetting::DetectorDef==3) // successive grain packings (detector 3)
  {  
   G4int nZ =0;
   for(G4int i=0;i<ParametersSetting::NbVoxels;i++)
   {
    if((ParametersSetting::XposId[i]==ParametersSetting::XposId[ParametersSetting::DetectorVox])
      &&(ParametersSetting::YposId[i]==ParametersSetting::YposId[ParametersSetting::DetectorVox]))
    {
     for(G4int j=0;j< int(ParametersSetting::DimVox[2]/ParametersSetting::DimDet[2]);j++)
     {
      map<G4int,G4double> E;
      for(std::map<G4int,G4double>::iterator
          it=DetectorParallelWorld::DiamVoxMap[i].begin();
          it!=DetectorParallelWorld::DiamVoxMap[i].end();++it)
      E[it->first]=0.;// creation of a map with the size for recording the dose in the packing box nZ
                    
      DetE_2_3.push_back(E);// reserve the size of the map in the vector DetE_2_3

      for(std::map<G4int,G4double>::iterator
      it=DetectorParallelWorld::DiamVoxMap[i].begin();
      it!=DetectorParallelWorld::DiamVoxMap[i].end();++it)
      DetE_2_3[nZ][it->first] =0.; // map initialization
			
      nZ++; // incrementing the packing box index
     }
    }
   }
  }
 }  // end of re-initialization
}

void RunAction::EndOfRunAction(const G4Run*)
{ 
 if(end) // end of the simulation, controlled by PrimaryGeneratorAction
 {
  G4cout<<G4endl<<"end of the simulation"<<G4endl;

  // writting the total energy loss by killing stucked particles
  G4String ErrorFile = "results/"+ParametersSetting::ResultName+"_Errors";
  ofstream Error;
  Error.open(ErrorFile, ios::out|ios::app);
  Error<<G4endl<<"Total number of particles killed:"<<G4endl<<NbPartkilled
  <<G4endl<<"Total energy loss by killing stucked particles in percent of the total energy of the emitted particles:"<<G4endl
  <<Eloss*100/InfMatE<<" %"<<G4endl;
  Error.close();

  // setting the result file names
  G4String RName ="results/";
  if(ParametersSetting::DetectorDef==1)RName += ParametersSetting::ResultName+"_Detector1";
  if(ParametersSetting::DetectorDef==2)RName += ParametersSetting::ResultName+"_Detector2";
  if(ParametersSetting::DetectorDef==3)RName += ParametersSetting::ResultName+"_Detector3";
  G4String PName = "results/"+ParametersSetting::ResultName+"_Probe";
  remove(PName);
  remove(RName);

  //
  // Probe results
  //
  G4cout<<"writing the probe results"<<G4endl;
  ofstream Res;
  Res.open(PName, ios::out|ios::app);
  Res<<"depth(mm)"<<" "<<"Material_Num"<<" "<<"Material_Name"<<" "<<"Density(g/cm3)"<<" "<<"Water_content(%)"
  <<" "<<"EmMass(Gray)"<<" "<<"Dose_absorbed(Gray)"<<G4endl;

  for(G4int i=0; i<ParametersSetting::NbProbeSegment;i++)
  {
   G4double DimSeg = ParametersSetting::DimSample[2]/ParametersSetting::NbProbeSegment;
   G4double PosZ = (0.5+i)*DimSeg;

   G4int x = int(ParametersSetting::XposId[ParametersSetting::DetectorVox]+0.5+ParametersSetting::ProbeOffSetXY[0]/ParametersSetting::DimVox[0]);
   G4int y = int(ParametersSetting::YposId[ParametersSetting::DetectorVox]+0.5+ParametersSetting::ProbeOffSetXY[1]/ParametersSetting::DimVox[1]);
   G4int z = int(PosZ/ParametersSetting::DimVox[2]);
   G4int fVoxelNum= int(x + ParametersSetting::NbVox_byAxis[0]*y + ParametersSetting::NbVox_byAxis[0]*ParametersSetting::NbVox_byAxis[1]*z);
 
   G4double fdepth = (i+0.5)*ParametersSetting::DimSample[2]/ParametersSetting::NbProbeSegment;        
   G4int fProb_MatNum = ParametersSetting::MaterialMap[fVoxelNum];
   G4String fProb_MatName = ParametersSetting::MatName[fProb_MatNum];
   G4double fProb_Densi = ParametersSetting::MatDensity[fProb_MatNum];

   if((fVoxelNum == ParametersSetting::DetectorVox)&&(ParametersSetting::DetectorDef==1))
   {
    x = int((0.5*ParametersSetting::DimVox[0]+ParametersSetting::ProbeOffSetXY[0])/ParametersSetting::DimDet[0]);
    y = int((0.5*ParametersSetting::DimVox[1]+ParametersSetting::ProbeOffSetXY[1])/ParametersSetting::DimDet[1]);
    z = int((PosZ-z*ParametersSetting::DimVox[2])/ParametersSetting::DimDet[2]);
    G4int fSubVoxelNum = 
    int(x + ParametersSetting::Object_voxelNumberXYZ[0]*y + ParametersSetting::Object_voxelNumberXYZ[0]*ParametersSetting::Object_voxelNumberXYZ[1]*z);
    fProb_MatNum = ParametersSetting::Object_MaterialMap[fSubVoxelNum];
    fProb_MatName = ParametersSetting::MatName[fProb_MatNum];
    fProb_Densi = ParametersSetting::MatDensity[fProb_MatNum];
   }
   G4double fProb_SegMass = DimSeg*M_PI*pow(ParametersSetting::ProbeDiameter/2.,2.)*fProb_Densi*g/cm3;
   G4double fProb_SegDose = ProbeE[i]/fProb_SegMass;
   G4double fProb_SegEmitDose = EmissionParameters::Eemitted_Probe[i]/fProb_SegMass;
	
   Res<<fdepth/mm<<" "<<fProb_MatNum<<" "<<fProb_MatName<<" "<<fProb_Densi<<" "<<ParametersSetting::MatWaterContent[fProb_MatNum]
   <<" "<<fProb_SegEmitDose/gray<<" "<<fProb_SegDose/gray<<G4endl;
  }
  Res.close();

  //
  // Detector results 
  //
  if(ParametersSetting::DetectorDef>0) Res.open(RName, ios::out|ios::app);

  	// detector type 1

  if(ParametersSetting::DetectorDef==1)
  {
   G4cout<<"writing the sub-voxelised detector results"<<G4endl;
   G4double DetMass =0.;
   G4double WaterMass =0.;
   G4double E_moy = 0.;
   G4double DoseVoxel_moy=0.;
   G4double fVariance =0.;
   G4double total_dose =0;
   G4double infMat_dose =0;
   G4int fNbOfMaterial=0;
   vector<G4bool> fDetectorMaterial;fDetectorMaterial.reserve(ParametersSetting::NbMat);
   for(G4int i=0;i<ParametersSetting::NbMat;i++)fDetectorMaterial[i]=false;
   
   // results averaged on all the sub-voxelised voxel
   for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)
   {
    G4double VoxelMass = ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[i]]*g/cm3
                        *ParametersSetting::DimDet[0]*ParametersSetting::DimDet[1]*ParametersSetting::DimDet[2];
    DetMass += VoxelMass;
    WaterMass += VoxelMass*ParametersSetting::MatWaterContent[ParametersSetting::Object_MaterialMap[i]];
    E_moy += Det1E[i];
    DoseVoxel_moy += Det1E[i]/VoxelMass;
    infMat_dose +=  EmParam->InfMatrixE_Det[i];
    fDetectorMaterial[ParametersSetting::Object_MaterialMap[i]]=true;  
   }
   for(G4int i=0;i<ParametersSetting::NbMat;i++)if(fDetectorMaterial[i])fNbOfMaterial++;  
   
   DoseVoxel_moy /= ParametersSetting::Object_NbVoxel;
   total_dose = E_moy/DetMass;  
   infMat_dose /= DetMass;

   for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)
   {
    G4double VoxelMass = ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[i]]*g/cm3
                        *ParametersSetting::DimDet[0]*ParametersSetting::DimDet[1]*ParametersSetting::DimDet[2];
    fVariance+=pow(Det1E[i]/VoxelMass-DoseVoxel_moy,2.);
   }
   fVariance /= ParametersSetting::Object_NbVoxel;
   Res<<"Results for all the sub-voxelised detector: "<<G4endl;
   Res<<"Detector_dry_mass(g)"<<" "<<"Average_water_content(%)"<<G4endl
   <<DetMass/g<<" "<<WaterMass/DetMass<<"%"<<G4endl
   <<"Number_of_materials"<<" "<<"Number_of_voxels"<<G4endl
   <<fNbOfMaterial<<" "<<ParametersSetting::Object_NbVoxel<<G4endl
   <<"EmMass(Gray)"<<" "<<"Dose(Gray)"<<" "<<"Dose_STDEV_between_voxels(Gray)"<<G4endl
   <<infMat_dose/gray<<" "<<total_dose /gray<<" "<<pow(fVariance,0.5)/gray<<G4endl;
	    
   map<G4int,G4double> Dose_byMat;
   map<G4int,G4double> sqE_byMat;
   map<G4int,G4double> fVariance_byMat;
   map<G4int,G4double> EmMass_byMat;
   map<G4int,G4int> NbVoxel_byMat;

   for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)
   {
    std::map<G4int,G4double>::iterator it=Dose_byMat.find(ParametersSetting::Object_MaterialMap[i]);
    if(it == Dose_byMat.end())
    {
     Dose_byMat[ParametersSetting::Object_MaterialMap[i]]=0.;
     fVariance_byMat[ParametersSetting::Object_MaterialMap[i]]=0.;
     EmMass_byMat[ParametersSetting::Object_MaterialMap[i]]=0.;
     NbVoxel_byMat[ParametersSetting::Object_MaterialMap[i]]=0;
    } // maps initialization
    Dose_byMat[ParametersSetting::Object_MaterialMap[i]] += Det1E[i];
    EmMass_byMat[ParametersSetting::Object_MaterialMap[i]] += EmParam->InfMatrixE_Det[i];
    NbVoxel_byMat[ParametersSetting::Object_MaterialMap[i]] ++;
   }

   for(std::map<G4int,G4double>::iterator it=Dose_byMat.begin();it!=Dose_byMat.end();++it)
   {
    G4double VoxelMass = ParametersSetting::MatDensity[it->first]*g/cm3
                         *ParametersSetting::DimDet[0]
                         *ParametersSetting::DimDet[1]
                         *ParametersSetting::DimDet[2]; 
    Dose_byMat[it->first] = Dose_byMat[it->first]/NbVoxel_byMat[it->first]/VoxelMass;
    EmMass_byMat[it->first] = EmMass_byMat[it->first]/NbVoxel_byMat[it->first]/VoxelMass;		
   }

   for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)
   {       
    G4double VoxelMass = ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[i]]*g/cm3
                         *ParametersSetting::DimDet[0]
                         *ParametersSetting::DimDet[1]
                         *ParametersSetting::DimDet[2]; 
    fVariance_byMat[ParametersSetting::Object_MaterialMap[i]] += 
    pow(Det1E[i]/VoxelMass-Dose_byMat[ParametersSetting::Object_MaterialMap[i]],2.);
   }

   // results averaged by material in the sub-voxelised voxel
   Res<<G4endl<<"Results by material: "<<G4endl;
   Res<<"MatNo MatName Nb_of_voxels Density(g/cm3) Water_content(%) EmMass(Gray) Dose(Gray) STDEV(Gray)" <<G4endl;

   for(std::map<G4int,G4double>::iterator
       it=Dose_byMat.begin();
       it!=Dose_byMat.end();++it)
   {
    fVariance_byMat[it->first] /= NbVoxel_byMat[it->first];
    G4double STDEV_byMat = pow(fVariance_byMat[it->first],0.5);
    Res<<it->first<<" "<<ParametersSetting::MatName[it->first]<<" "<<NbVoxel_byMat[it->first]
    <<" "<<ParametersSetting::MatDensity[it->first]<<" "<<ParametersSetting::MatWaterContent[it->first]
    <<" "<<EmMass_byMat[it->first]/gray<<" "<<Dose_byMat[it->first]/gray<<" "<<STDEV_byMat/gray<<G4endl;
   }
   Res<<G4endl;

   // definition of the dose mapping parameters and equation
   G4double Dmin =Det1E[0]/(ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[0]]*g/cm3
                  *ParametersSetting::DimDet[0]
                  *ParametersSetting::DimDet[1]
                  *ParametersSetting::DimDet[2]);
   G4double Dmax =0.;
   if(ParametersSetting::MatOfInterest < 0)
   {
    for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)
    {
     G4double fDose = Det1E[i]/(ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[i]]*g/cm3
                      *ParametersSetting::DimDet[0]
                      *ParametersSetting::DimDet[1]
                      *ParametersSetting::DimDet[2]);
     if(fDose>Dmax)Dmax=fDose;	// maximum of energy
     if(fDose<Dmin)Dmin=fDose;	// minimum of energy
    }
   } 	
   else
   {
    for(G4int i=0;i<ParametersSetting::Object_NbVoxel;i++)
    {
     G4double fDose = Det1E[i]/(ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[i]]*g/cm3
                      *ParametersSetting::DimDet[0]
                      *ParametersSetting::DimDet[1]
                      *ParametersSetting::DimDet[2]);
     if((fDose>Dmax)&&(ParametersSetting::Object_MaterialMap[i]==ParametersSetting::MatOfInterest))Dmax=fDose;	// maximum of energy
     if((fDose<Dmin)&&(ParametersSetting::Object_MaterialMap[i]==ParametersSetting::MatOfInterest))Dmin=fDose;	// minimum of energy
    } 
   }
   Res<<"DoseMapping information: "<<G4endl
   <<"Maximum dose: Dmax = "<<Dmax/gray<<" Gray"<<G4endl
   <<"Minimum dose: Dmin = "<<Dmin/gray<<" Gray"<<G4endl;
   Res<<"Equation for grey value conversion into dose in DoseMapping: dose(Gray)= (grey_value-1) x (Dmax(Gray)-Dmin(Gray))/65534 + Dmin(Gray)"<<G4endl;

   // Dose mapping writting
   for(G4int j=0;j<ParametersSetting::Object_voxelNumberXYZ[2];j++)
   {
    G4String SliceName = "results/DoseMapping/"+ParametersSetting::ResultName;

    ostringstream oss; oss<< j;
    G4int k=3;
    if((j>=10)&&(j<100))k=2;
    if((j>=100)&&(j<1000))k=1;
    if((j>=1000)&&(j<10000))k=0;
    for(G4int l=0;l<k;l++)SliceName += "0";
    SliceName +=  oss.str()+".txt"; 	    // setting the slice dial on the slice name

    remove(SliceName);
    ofstream fmap;
    fmap.open(SliceName.c_str(),ios::out);

    for(G4int i=0;i<ParametersSetting::Object_voxelNumberXYZ[0]*ParametersSetting::Object_voxelNumberXYZ[1];i++)
    {
     G4int I_x = i%ParametersSetting::Object_voxelNumberXYZ[0];
     G4int I_y = ((i-I_x)/ParametersSetting::Object_voxelNumberXYZ[0])%ParametersSetting::Object_voxelNumberXYZ[1];
     if((I_x==0)&&(i>0))fmap<<endl;
     if((I_x==0)&&(I_y==0)&&(i>0))fmap<<endl; // navigation in the slice file
	       
     G4int Int = i + j*ParametersSetting::Object_voxelNumberXYZ[0]*ParametersSetting::Object_voxelNumberXYZ[1];
     G4double fDose = Det1E[Int]/(ParametersSetting::MatDensity[ParametersSetting::Object_MaterialMap[Int]]*g/cm3
                                *ParametersSetting::DimDet[0]
                                *ParametersSetting::DimDet[1]
                                *ParametersSetting::DimDet[2]);
     G4double fvalue = int((fDose-Dmin)/(Dmax-Dmin)*65534.+1); // translating the dose in a grey value

     if(ParametersSetting::MatOfInterest < 0)fmap<<fvalue<<" "; // dose mapping for all the materials in the detector
     else // dose mapping only for  the material of interest in the detector
     {
      if(ParametersSetting::Object_MaterialMap[i + j*ParametersSetting::Object_voxelNumberXYZ[0]*ParametersSetting::Object_voxelNumberXYZ[1]]
        ==ParametersSetting::MatOfInterest)
      fmap<<fvalue<<" ";
      else fmap<<0<<" ";
     }
    }
   }
  } // end of writing the results for the detector 1

  	// detector type 2

  if(ParametersSetting::DetectorDef==2)
  {
   // result by grain diameter
   G4cout<<"writing the single packing of grains detector results"<<G4endl;
   Res<<"Voxel_No Diam(mm) Nb_of_grains Density(g/cm3) EmMass(Gray) Dose(Gray) STDEV(Gray)"<<G4endl;

   map<G4double,G4double> E_byDiameter;
   map<G4double,G4int> nGrain_byDiameter;
   map<G4double,G4double> Variance_byDiameter;

   for(std::map<G4int,G4double>::iterator it=DetE_2_3[0].begin();it!=DetE_2_3[0].end();++it)
   {
    std::map<G4double,G4double>::iterator iter
    =E_byDiameter.find(DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]);
    if(iter == E_byDiameter.end())
    {
     E_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]]=0.;
     nGrain_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]]=0;
     Variance_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]]=0;
    }
    E_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]]+=DetE_2_3[0][it->first];
    nGrain_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]] ++ ;
   }

   for(std::map<G4double,G4double>::iterator it=E_byDiameter.begin(); it!=E_byDiameter.end();++it)
   E_byDiameter[it->first] = E_byDiameter[it->first] / nGrain_byDiameter[it->first]; // average

   for(std::map<G4int,G4double>::iterator it=DetE_2_3[0].begin(); it!=DetE_2_3[0].end();++it)
   {
    G4double GrainMass = pow((DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first])/2.,3.)*4./3.*M_PI*g/cm3
                         *ParametersSetting::Graindensity[ParametersSetting::MaterialMap[ParametersSetting::DetectorVox]];
    Variance_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]]+= 
    pow(DetE_2_3[0][it->first]/GrainMass
    -E_byDiameter[DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]]/GrainMass,2.);
   }

   for(std::map<G4double,G4double>::iterator it=Variance_byDiameter.begin(); it!=Variance_byDiameter.end();++it)
   Variance_byDiameter[it->first]= Variance_byDiameter[it->first]/nGrain_byDiameter[it->first]; // variance

   for(std::map<G4double,G4double>::iterator it=E_byDiameter.begin(); it!=E_byDiameter.end();++it)
   {
    G4double GrainMass = pow((it->first)/2.,3.)*4./3.*M_PI*g/cm3
                         *ParametersSetting::Graindensity[ParametersSetting::MaterialMap[ParametersSetting::DetectorVox]];
    G4double DetMass = ParametersSetting::MatDensity[ParametersSetting::MaterialMap[ParametersSetting::DetectorVox]]*g/cm3
                       *ParametersSetting::DimVox[0]*ParametersSetting::DimVox[1]*ParametersSetting::DimVox[2];
    G4double Dose = E_byDiameter[it->first]/GrainMass;
    G4double STDEV = pow(Variance_byDiameter[it->first],0.5);
    G4double InfiniteMatrixDose = EmissionParameters::InfMatrixE_Det[0]/DetMass;
                        
    Res<<ParametersSetting::DetectorVox <<" "<<(it->first)/mm<<" "<<nGrain_byDiameter[it->first]<<" "
    <<ParametersSetting::Graindensity[ParametersSetting::MaterialMap[ParametersSetting::DetectorVox]]
    <<" "<<InfiniteMatrixDose/gray<<" "<<Dose/gray<<" "<<STDEV/gray<<G4endl;
   }

   Res.close();

   // result for each grain
   RName = "results/"+ParametersSetting::ResultName + "_grains";
   remove(RName);
   Res.open(RName, ios::out|ios::app);
   Res<<"Voxel_No Grain_No diam(mm) density(g/cm3) Dose(gray)"<<G4endl;

   for(std::map<G4int,G4double>::iterator it=DetE_2_3[0].begin(); it!=DetE_2_3[0].end();++it)
   {         
    G4double GrainMass = pow(DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]/2.,3.)*4./3.*M_PI*g/cm3
                         *ParametersSetting::Graindensity[ParametersSetting::MaterialMap[ParametersSetting::DetectorVox]];
    G4double Dose = DetE_2_3[0][it->first]/GrainMass;

    Res<<ParametersSetting::DetectorVox<<" "<<it->first<<" "<<DetectorParallelWorld::DiamVoxMap[ParametersSetting::DetectorVox][it->first]
    <<" "<<ParametersSetting::Graindensity[ParametersSetting::MaterialMap[ParametersSetting::DetectorVox]]<<" "<<Dose/gray<<G4endl;
   }
  } // end of writing the results for the detector 2

  	// detector type 3

  if(ParametersSetting::DetectorDef==3)
  {
   // result by packing box and grain diameter
   G4cout<<"writing the successive packings of grains detector results"<<G4endl;
   Res<<"Voxel_No Zdiv_No Diam(mm) Nb_of_grains Density(g/cm3) EmMass(Gray) Dose(Gray) STDEV(Gray)"<<G4endl;

   G4int nZ =0;
   for(G4int i=0;i<ParametersSetting::NbVoxels;i++)
   {
    if((ParametersSetting::XposId[i]==ParametersSetting::XposId[ParametersSetting::DetectorVox])
      &&(ParametersSetting::YposId[i]==ParametersSetting::YposId[ParametersSetting::DetectorVox]))
    {
     for(G4int j=0;j< int(ParametersSetting::DimVox[2]/ParametersSetting::DimDet[2]);j++)
     {
      map<G4double,G4double> E_byDiameter;
      map<G4double,G4int> nGrain_byDiameter;
      map<G4double,G4double> Variance_byDiameter;

      for(std::map<G4int,G4double>::iterator it=DetE_2_3[nZ].begin(); it!=DetE_2_3[nZ].end();++it)
      {
       std::map<G4double,G4double>::iterator iter=E_byDiameter.find(DetectorParallelWorld::DiamVoxMap[i][it->first]);
       if(iter == E_byDiameter.end())
       {
        E_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]]=0.;
        nGrain_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]]=0;
	Variance_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]]=0;
       } // maps initialization
       E_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]]+=DetE_2_3[nZ][it->first];
       nGrain_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]] ++ ;
      }

      for(std::map<G4double,G4double>::iterator it=E_byDiameter.begin(); it!=E_byDiameter.end();++it)
      E_byDiameter[it->first] = E_byDiameter[it->first] / nGrain_byDiameter[it->first]; // average

      for(std::map<G4int,G4double>::iterator it=DetE_2_3[nZ].begin(); it!=DetE_2_3[nZ].end();++it)
      {
       G4double GrainMass = pow((DetectorParallelWorld::DiamVoxMap[i][it->first])/2.,3.)*4./3.*M_PI*g/cm3
                            *ParametersSetting::Graindensity[ParametersSetting::MaterialMap[i]];
       Variance_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]]+= 
       pow(DetE_2_3[nZ][it->first]/GrainMass-E_byDiameter[DetectorParallelWorld::DiamVoxMap[i][it->first]]/GrainMass,2.);
      }

      for(std::map<G4double,G4double>::iterator it=Variance_byDiameter.begin(); it!=Variance_byDiameter.end();++it)
      Variance_byDiameter[it->first]= Variance_byDiameter[it->first]/nGrain_byDiameter[it->first]; // variance

      for(std::map<G4double,G4double>::iterator it=E_byDiameter.begin(); it!=E_byDiameter.end();++it)
      {
       G4double GrainMass = pow((it->first)/2.,3.)*4./3.*M_PI*g/cm3*ParametersSetting::Graindensity[ParametersSetting::MaterialMap[i]];
       G4double DetMass = ParametersSetting::MatDensity[ParametersSetting::MaterialMap[i]]*g/cm3
                          *ParametersSetting::DimVox[0]*ParametersSetting::DimVox[1]*ParametersSetting::DimVox[2];
       G4double Dose = E_byDiameter[it->first]/GrainMass;
       G4double STDEV = pow(Variance_byDiameter[it->first],0.5);
       G4double InfiniteMatrixDose = EmissionParameters::InfMatrixE_Det[ParametersSetting::ZposId[i]]/DetMass;
                        
       Res<<i<<" "<<nZ<<" "<<(it->first)/mm<<" "<<nGrain_byDiameter[it->first]<<" "
       <<ParametersSetting::Graindensity[ParametersSetting::MaterialMap[i]]<<" "
       <<InfiniteMatrixDose/gray<<" "<<Dose/gray<<" "<<STDEV/gray<<G4endl;
      }                 
      nZ++;
     }
    }
   }

   Res.close();

   // result for each grain
   RName = "results/"+ParametersSetting::ResultName + "_grains";
   remove(RName);
   Res.open(RName, ios::out|ios::app);
   Res<<"Voxel_No Zdiv_No Grain_No diam(mm) density(g/cm3) Dose(gray)"<<G4endl;

   nZ =0;
   for(G4int i=0;i<ParametersSetting::NbVoxels;i++)
   {
    if((ParametersSetting::XposId[i]==ParametersSetting::XposId[ParametersSetting::DetectorVox])
      &&(ParametersSetting::YposId[i]==ParametersSetting::YposId[ParametersSetting::DetectorVox]))
    {
     for(G4int j=0;j< int(ParametersSetting::DimVox[2]/ParametersSetting::DimDet[2]);j++)
     {
      for(std::map<G4int,G4double>::iterator it=DetE_2_3[nZ].begin(); it!=DetE_2_3[nZ].end();++it)
      {
       G4double GrainMass = pow(DetectorParallelWorld::DiamVoxMap[i][it->first]/2.,3.) *4./3.*M_PI*g/cm3
                            *ParametersSetting::Graindensity[ParametersSetting::MaterialMap[i]];
       G4double Dose = DetE_2_3[nZ][it->first]/GrainMass;

       Res<<i<<" "<<nZ<<" "<<it->first<<" " <<DetectorParallelWorld::DiamVoxMap[i][it->first]
       <<" "<<ParametersSetting::Graindensity[ParametersSetting::MaterialMap[i]]<<" "<<Dose/gray<<G4endl;
      }
      nZ++;
     }
    }
   }
  } // end of writing the results for the detector 3

  Res.close();
 } // end of results writing

}


