//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "SteppingAction.hh"

using namespace std;

SteppingAction::SteppingAction(RunAction* run,PrimaryGeneratorAction* primG)
:RunAct(run),myPrimGenAction(primG)
{
 // getting geometry and detector data for faster access
 DetVox = ParametersSetting::DetectorVox;
 DetDef = ParametersSetting::DetectorDef; // detector data

 ProbeRadius = ParametersSetting::ProbeDiameter/2.;
 POffSetX = ParametersSetting::ProbeOffSetXY[0];
 POffSetY = ParametersSetting::ProbeOffSetXY[1]; // cylindrical probe data

 fNvoxX = ParametersSetting::NbVox_byAxis[0];
 fNvoxY = ParametersSetting::NbVox_byAxis[1];

 fDetvoxX = 0;
 fDetvoxY = 0;
 if(ParametersSetting::DetectorDef==1)
 {
  fDetvoxX = ParametersSetting::Object_voxelNumberXYZ[0];
  fDetvoxY = ParametersSetting::Object_voxelNumberXYZ[1];
 }

 E_lost = 0.;
 security = 0;
 VoxelSafety = true;

 // initialization of the errors file
 ErrorFile = "results/"+ParametersSetting::ResultName+"_Errors";
 remove(ErrorFile);
 ofstream Error;
 Error.open(ErrorFile, ios::out|ios::app);
 Error<<"list of the particles killed because of an infinite loop in path calculation:"<<G4endl
 <<"particle remaining_energy(keV) positionX(mm) positionY(mm) positionZ(mm) object object_number object_material"<<G4endl;
 Error.close();
}

SteppingAction::~SteppingAction()
{;}

void SteppingAction::UserSteppingAction(const G4Step* step)
{
 point1 = step->GetPreStepPoint();

 //
 // correction of infinite loop during navigation
 //

 if((step->GetStepLength()/micrometer==0.)&&(step->GetTotalEnergyDeposit()/eV==0.)&&(VoxelSafety))
 {
  if(point1->GetKineticEnergy()/keV>0.)
  {  
   security ++;
   if(security>5)
   {
    VoxelSafety = false;   
   }
   else
   {
    G4ThreeVector new_position = point1->GetPosition();
    new_position.setX(point1->GetPosition().getX()+(G4UniformRand()-0.5)*ParametersSetting::DimDet[0]/1000.);
    new_position.setY(point1->GetPosition().getY()+(G4UniformRand()-0.5)*ParametersSetting::DimDet[1]/1000.);
    new_position.setZ(point1->GetPosition().getZ()+(G4UniformRand()-0.5)*ParametersSetting::DimDet[2]/1000.);
    point1->SetPosition(new_position); // small shift applied to the particle position to get out of the voxel boundaries
   } // correction for particle stucked between two voxels
  }
 }
 //else {VoxelSafety =true;}

 if(step->GetStepLength()/micrometer>0.){security=0;VoxelSafety =true;}

 if(security> 1000000)
 {
  E_lost = point1->GetKineticEnergy(); 
  RunAct->EnergyLoss(E_lost);
  ofstream Error;
  Error.open(ErrorFile, ios::out|ios::app);
  Error<<step->GetTrack()->GetDefinition()->GetParticleName()<<" "<<E_lost/keV<<" "
       <<point1->GetPosition().getX()/mm<<" "<<point1->GetPosition().getY()/mm<<" "<<point1->GetPosition().getZ()/mm<<" "
       <<point1->GetPhysicalVolume()->GetName()<<" "<<point1->GetPhysicalVolume()->GetCopyNo()<<" "
       <<point1->GetPhysicalVolume()->GetLogicalVolume()->GetMaterial()->GetName()<<G4endl;
 Error.close(); // registration of the killed particle data
 step->GetTrack()->SetTrackStatus(fStopAndKill);
} // correction for particle in a infinite loop of path calculation -> particle killed

 G4int VoxelNo = -1; 
 if(point1->GetPhysicalVolume()->GetName()=="Voxel")
 {
  G4int Z_No = point1->GetTouchable()->GetCopyNumber();
  G4int Y_No = point1->GetTouchable()->GetVolume(1)->GetCopyNo();
  G4int X_No = point1->GetTouchable()->GetVolume(2)->GetCopyNo();
  VoxelNo = X_No + Y_No*fNvoxX + Z_No*fNvoxX*fNvoxY;
 } // getting the voxel number in the main voxelised grid

 static G4PathFinder* sPathFinder = 0;
 if(!sPathFinder)sPathFinder = G4PathFinder::GetInstance();

 //
 // cylindrical probe dosimetry
 //
	// creation and initialization of the probe world navigator
 //static G4Navigator* ParaNavigator = 0;
static G4Navigator* ParaNavigator=0;
 if(!ParaNavigator)
 {
  G4String ProbWName = "ProbeWorld";
  G4VPhysicalVolume* pPW = G4TransportationManager::GetTransportationManager()->GetParallelWorld(ProbWName);
  ParaNavigator =G4TransportationManager::GetTransportationManager()->GetNavigator(pPW);
 } 
 ProbeWorldID = G4TransportationManager::GetTransportationManager()->ActivateNavigator(ParaNavigator);

	// getting the volume in the probe world at the particle location

 G4ThreeVector position = point1->GetPosition();
 G4ThreeVector momDirection = point1->GetMomentumDirection();

 sPathFinder->PrepareNewTrack(position,momDirection);
 G4TouchableHandle touchableProbW = sPathFinder-> CreateTouchableHandle(ProbeWorldID); 

	// recording the energy deposited if the volume is one of the probe segments
 if(touchableProbW->GetVolume()->GetName() == "Probe")
 {
  G4int ProbSegNum = touchableProbW->GetVolume()->GetCopyNo();
  RunAct->AccumProbeEnergy(step->GetTotalEnergyDeposit(),ProbSegNum);
  //if(ProbSegNum==50)G4cout<<"step "<<step->GetTotalEnergyDeposit()/keV<<G4endl;
 }

G4StepPoint* point2 = step->GetPostStepPoint();

 //
 // Detector dosimetry
 //
 if((DetDef>0)&&(VoxelNo>=0)) // if a detector over than the cylindrical probe has been created
 {
  	// creation and initialization of the detector world navigator
  static G4Navigator* DetNavigator = 0;
  if(!DetNavigator)
  {
   G4String DetWName = "DetectorWorld";
   G4VPhysicalVolume* dPW = G4TransportationManager::GetTransportationManager()->GetParallelWorld(DetWName);
   DetNavigator =G4TransportationManager::GetTransportationManager()->GetNavigator(dPW);
  }

  G4int DetectorWorldID = G4TransportationManager::GetTransportationManager()->ActivateNavigator(DetNavigator);
  sPathFinder->PrepareNewTrack(point1->GetPosition(),point1->GetMomentum());
  G4TouchableHandle touchableDetW = sPathFinder-> CreateTouchableHandle(DetectorWorldID );

  	// sub-voxelised detector dosimetry (detector 1)
  if((DetDef==1)&&(DetVox==VoxelNo))
  {
   if(touchableDetW->GetVolume())
   {
    DetMat = 0;
    DetMat = touchableDetW->GetVolume()->GetLogicalVolume()->GetMaterial();
    if(DetMat)
    {
     G4double Energy = step->GetTotalEnergyDeposit();
     G4int Z_DetNo = touchableDetW->GetCopyNumber();
     G4int Y_DetNo = touchableDetW->GetVolume(1)->GetCopyNo();
     G4int X_DetNo = touchableDetW->GetVolume(2)->GetCopyNo();// getting the copy number of the successive parametrizations (nested parameterization)

     G4int DetVoxNo = X_DetNo + Y_DetNo*fDetvoxX + Z_DetNo*fDetvoxX*fDetvoxY;// calculating the voxel number
     RunAct->AccumDetectorEnergy(Energy,DetVoxNo); // recording the dose for the corresponding voxel
    }
   }
  }

  	// single packing of grains detector dosimetry (detector 2)
  if((DetDef==2)&&(DetVox==VoxelNo))
  {
   if(touchableDetW->GetVolume()) // testing if a volume in the detector world exists at the particle location
   { 
    // testing if the volume at the particle location is a spherical shaped grain (valid detector)
    if(touchableDetW->GetVolume()->GetLogicalVolume()->GetSolid()->GetName()=="grain")
    {
     G4double E = step->GetTotalEnergyDeposit();
     G4int grainNo = touchableDetW->GetVolume()->GetCopyNo(); // getting the grain number
     RunAct->AccumGrainEnergy(E,0,grainNo); // recording the dose for the corresponding grain
    }
   }
  }

  	// successive packings of grains detector dosimetry (detector 3)
  if((DetDef==3)
    &&(ParametersSetting::XposId[VoxelNo]==ParametersSetting::XposId[DetVox])
    &&(ParametersSetting::YposId[VoxelNo]==ParametersSetting::YposId[DetVox]))
  {
   if(touchableDetW->GetVolume()) // testing if a volume in the detector world exists at the particle location
   {
    // testing if the volume at the particle location is a spherical shaped grain (valid detector)
    if(touchableDetW->GetVolume()->GetLogicalVolume()->GetSolid()->GetName()=="grain")
    {
     G4double E = step->GetTotalEnergyDeposit();
     G4int grainNo = touchableDetW->GetVolume()->GetCopyNo();  // getting the grain number
     G4int BoxNo = touchableDetW->GetVolume(1)->GetCopyNo(); // getting the packing box number
     RunAct->AccumGrainEnergy(E,BoxNo,grainNo);// recording the dose for the corresponding grain and box
    }
   }
  }

 } // end of the detector dosimetry
sPathFinder->Locate(point2->GetPosition(),point2->GetMomentumDirection());


// code lines for particle reflexion at the boundaries of the model
/*
 G4bool reflexionX = 1; 
 G4bool reflexionY = 1; 
 G4bool reflexionZ = 1; 


 
  if((point1->GetPhysicalVolume()->GetName() != "World")&&(point2->GetPhysicalVolume()))
  {
  // G4StepPoint* point2 = step->GetPostStepPoint();
   if(point2->GetPhysicalVolume()->GetName() == "World")
   {
    if((!reflexionX)&&(!reflexionY)&&(!reflexionZ))
    step->GetTrack()->SetTrackStatus(fStopAndKill);// killing the particle if it exits the sample
    else
    {
     G4ThreeVector fmomentum = point2->GetMomentumDirection();
     G4double fdifX = fabs(fabs(point2->GetPosition().getX()) - ParametersSetting::DimSample[0]/2.);
     G4double fdifY = fabs(fabs(point2->GetPosition().getY()) - ParametersSetting::DimSample[1]/2.);
     G4double fdifZ = fabs(fabs(point2->GetPosition().getZ()) - ParametersSetting::DimSample[2]/2.);

     if((fdifX<=fdifY)&&(fdifX<=fdifZ))
     {
      if(reflexionX)
      {    
       G4ThreeVector fNewMomentum((-1.)*fmomentum.getX(),fmomentum.getY(),fmomentum.getZ());
       point2->SetMomentumDirection(fNewMomentum);     
      }
      else step->GetTrack()->SetTrackStatus(fStopAndKill);
     }
   

    if((fdifY<=fdifX)&&(fdifY<=fdifZ))
    {
     if(reflexionY)
     {
      G4ThreeVector fNewMomentum(fmomentum.getX(),(-1.)*fmomentum.getY(),fmomentum.getZ());
      point2->SetMomentumDirection(fNewMomentum);    
     }
     else step->GetTrack()->SetTrackStatus(fStopAndKill);
    }

    if((fdifZ<=fdifX)&&(fdifZ<=fdifY))
    {
     if(reflexionZ)
     {
      G4double fMomZ = point2->GetMomentumDirection().getZ();
      G4ThreeVector fNewMomentum(point2->GetMomentumDirection().getX(),point2->GetMomentumDirection().getY(),(-1.)*fMomZ);
      point2->SetMomentumDirection(fNewMomentum);
     }
     else step->GetTrack()->SetTrackStatus(fStopAndKill);
    }
   }
  }
 }
*/
}
 

