//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "VoxParamXY.hh"

VoxParamXY::VoxParamXY(G4int axis, G4int world)
{ 
 fAxisFlag = axis;
 fWorldFlag = world; // getting the axis and world ID
 
 if(fWorldFlag == 0) // size of the divisions of the main voxel grid
 {
  fVoxelSize[0]=ParametersSetting::DimVox[0];
  fVoxelSize[1]=ParametersSetting::DimVox[1];
  fGridSize[0]=ParametersSetting::DimSample[0];
  fGridSize[1]=ParametersSetting::DimSample[1];
 }
 else  // size of the divisions of the sub-voxelised detector
 {
  fVoxelSize[0]=ParametersSetting::DimDet[0];
  fVoxelSize[1]=ParametersSetting::DimDet[1];
  fGridSize[0]=ParametersSetting::DimVox[0];
  fGridSize[1]=ParametersSetting::DimVox[1];
 }
}

VoxParamXY::~VoxParamXY()
{;}

void VoxParamXY::ComputeTransformation(const G4int copyNo,G4VPhysicalVolume *physVol) const
{
 G4ThreeVector Position((fVoxelSize[fAxisFlag]*(copyNo+0.5)-fGridSize[fAxisFlag]/2.)*(1-fAxisFlag),
                        (fVoxelSize[fAxisFlag]*(copyNo+0.5)-fGridSize[fAxisFlag]/2.)*(fAxisFlag),
                        0);
 physVol->SetTranslation(Position); // calculating and setting the position of each segment
}


